from mock import Mock, patch, call
from tests.ditest import DependencyInjectionTestBase


class FormatFactoryTests(DependencyInjectionTestBase):

    def setUp(self):
        super(FormatFactoryTests, self).setUp()

    def test_Provides_json(self):
        from niprov.formatfactory import FormatFactory
        from niprov.formatjson import JsonFormat
        factory = FormatFactory()
        self.assertIsInstance(factory.create('json'), JsonFormat)

    def test_Provides_xml(self):
        from niprov.formatfactory import FormatFactory
        from niprov.formatxml import XmlFormat
        factory = FormatFactory()
        self.assertIsInstance(factory.create('xml'), XmlFormat)

    def test_Provides_narrated(self):
        from niprov.formatfactory import FormatFactory
        from niprov.formatnarrated import NarratedFormat
        factory = FormatFactory()
        self.assertIsInstance(factory.create('narrated'), NarratedFormat)

    def test_Provides_simple(self):
        from niprov.formatfactory import FormatFactory
        from niprov.formatsimple import SimpleFormat
        factory = FormatFactory()
        self.assertIsInstance(factory.create('simple'), SimpleFormat)

    def test_Provides_dict(self):
        from niprov.formatfactory import FormatFactory
        from niprov.formatdict import DictFormat
        factory = FormatFactory()
        self.assertIsInstance(factory.create('dict'), DictFormat)

    def test_Provides_object(self):
        from niprov.formatfactory import FormatFactory
        from niprov.formatobject import ObjectFormat
        factory = FormatFactory()
        self.assertIsInstance(factory.create('object'), ObjectFormat)

    def test_Provides_picture(self):
        from niprov.formatfactory import FormatFactory
        from niprov.pictures import PictureCache
        factory = FormatFactory()
        self.assertIsInstance(factory.create('picture'), PictureCache)

    def test_Raises_exception_on_unknown_name(self):
        from niprov.formatfactory import FormatFactory
        factory = FormatFactory()
        with self.assertRaisesRegexp(ValueError, 'Unknown format: hexameter'):
           factory.create('hexameter')







from niprov.diff import Diff


def compare(file1, file2, dependencies=None):
    """Compare the provenance of two files.

    This creates a niprov :class:`.Diff` object that can be further 
    interrogated or displayed for differences between the two files.

    Example:
        Chain calls with methods of Diff or print the Diff object::

            compare(file1, file2).assertEqual(ignore='path')
            print(compare(file1, file2))

    Args:
        file1 (:class:`.BaseFile`): One of two niprov BaseFile objects to compare.
        file2 (:class:`.BaseFile`): As file1

    Returns:
        niprov.diff.Diff: A niprov :class:`.Diff` object which reflects
            differences between the two files.
    """
    return Diff(file1, file2)



import socket, os.path


class Location(object):
    """Represents the location of a file."""

    def __init__(self, locationString):
        if ':' in locationString:
            parts = locationString.split(':')
            self.hostname = parts[0]
            path = parts[1]
        else:
            self.hostname = socket.gethostname()
            path = locationString
        self.path = os.path.abspath(path)

    def toDictionary(self):
        d = {}
        d['path'] = self.path
        d['hostname'] = self.hostname
        d['location'] = str(self)
        return d

    def toUrl(self):
        return 'file://{0}{1}'.format(self.hostname, self.path)

    def __str__(self):
        return self.toString()

    def toString(self):
        return ':'.join([self.hostname, self.path])

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return str(self) == str(other)
        else:
            return False

    def __ne__(self, other):
        return not self.__eq__(other)



from niprov.dependencies import Dependencies
from niprov.formatjson import JsonFormat
from niprov.formatxml import XmlFormat
from niprov.formatnarrated import NarratedFormat
from niprov.formatsimple import SimpleFormat
from niprov.formatdict import DictFormat
from niprov.formatobject import ObjectFormat
from niprov.pictures import PictureCache


class FormatFactory(object):

    def __init__(self, dependencies=Dependencies()):
        self.dependencies = dependencies

    def create(self, formatName):
        if formatName == 'json':
            return JsonFormat(self.dependencies)
        if formatName == 'xml':
            return XmlFormat(self.dependencies)
        if formatName == 'narrated':
            return NarratedFormat()
        if formatName == 'simple':
            return SimpleFormat()
        if formatName == 'dict':
            return DictFormat()
        if formatName == 'object':
            return ObjectFormat()
        if formatName == 'picture':
            return PictureCache(self.dependencies)
        raise ValueError('Unknown format: '+str(formatName))



from niprov.format import Format


class ObjectFormat(Format):

    def serializeSingle(self, provenance):
        return provenance

    def serializeList(self, provenance):
        return provenance

    def serializeStatistics(self, provenance):
        return provenance

    def serializePipeline(self, provenance):
        return provenance



#!/usr/bin/python
# -*- coding: UTF-8 -*-
import unittest
from mock import Mock
from datetime import datetime
import json
from tests.ditest import DependencyInjectionTestBase


class SerializerTests(DependencyInjectionTestBase):

    def setUp(self):
        super(SerializerTests, self).setUp()
        self.fileFactory.fromProvenance.side_effect = lambda p: self.imageWithProvenance(p)

    def imageWithProvenance(self, prov):
        img = Mock()
        img.provenance = prov
        if 'location' in prov:
            img.location.toString.return_value = prov['location']
        return img

    def test_serialize_makes_args_kwargs_values_strings(self):
        from niprov.formatjson import JsonFormat  
        class CustomType(object):
            def __str__(self):
                return '<CustomType object>'
        serializer = JsonFormat(self.dependencies)
        record = {}
        ct1 = CustomType()
        ct2 = CustomType()
        ct3 = CustomType()
        record['args'] = [1.23, ct1]
        record['kwargs'] = {'one':ct2, 'two':4.56}
        record['_versions'] = [{'args':[7.89, ct3]}]
        out = serializer.serializeSingle(self.imageWithProvenance(record))
        self.assertEqual(json.loads(out)['args'], 
            [1.23, str(ct1)])
        self.assertEqual(json.loads(out)['kwargs'], 
            {'one':str(ct2), 'two':4.56})

    def test_serialize_and_deserialize_dont_balk_if_time_field_absent(self):
        from niprov.formatjson import JsonFormat  
        serializer = JsonFormat(self.dependencies)
        record = {}
        img1 = self.imageWithProvenance(record)
        jsonrecord = serializer.serializeSingle(img1)
        out = serializer.deserialize(jsonrecord)
        self.assertEqual(img1.provenance, out.provenance)

    def test_swallows_object_ids(self):
        from bson.objectid import ObjectId
        from niprov.formatjson import JsonFormat  
        serializer = JsonFormat(self.dependencies)
        record = {}
        record['_id'] = ObjectId('564168f2fb481f480891263c')
        record['_versions'] = [{'_id':ObjectId('564168f2fb481f480891263c')}]
        out = serializer.serializeList([self.imageWithProvenance(record)])
        self.assertNotIn('_id', json.loads(out)[0])
        self.assertNotIn('_id', json.loads(out)[0]['_versions'][0])

    def test_Deals_with_versions(self):
        from niprov.formatjson import JsonFormat  
        serializer = JsonFormat(self.dependencies)
        record = {}
        dtnow = datetime.now()
        record['_versions'] = [{'acquired':dtnow}, 
                              {'added':dtnow}]
        jsonStr = serializer.serializeSingle(self.imageWithProvenance(record))
        out = serializer.deserialize(jsonStr)
        self.assertEqual(out.provenance['_versions'][-1]['added'], dtnow)
        self.assertEqual(out.provenance['_versions'][-2]['acquired'], dtnow)





import unittest
import mock
from mock import Mock, patch


class DependenciesTests(unittest.TestCase):

    def setUp(self):
        pass

    def test_Changing_setting_directly_affects_component_setting(self):
        from niprov.dependencies import Dependencies
        dependencies = Dependencies()
        self.assertEqual(dependencies.getListener().verbosity, 'info')
        dependencies.config.verbosity = 'warning'
        self.assertEqual(dependencies.getListener().verbosity, 'warning')

    def test_Changing_storage_setting_changes_repository_provided(self):
        from niprov.dependencies import Dependencies
        from niprov.jsonfile import JsonFile
        from niprov.mongo import MongoRepository
        dependencies = Dependencies()
        with patch('niprov.mongo.pymongo'):
            dependencies.config.database_type = 'file'
            self.assertIsInstance(dependencies.getRepository(), JsonFile)
            dependencies.config.database_type = 'MongoDB'
            self.assertIsInstance(dependencies.getRepository(), MongoRepository)

    def test_reconfigureOrGetConfiguration_with_None_doesnt_affect_config(self):
        from niprov.dependencies import Dependencies
        dependencies = Dependencies()
        self.assertEqual(dependencies.getListener().verbosity, 'info')
        dependencies.reconfigureOrGetConfiguration(None)
        self.assertEqual(dependencies.getListener().verbosity, 'info')

    def test_reconfigureOrGetConfiguration_with_None_returns_config(self):
        from niprov.dependencies import Dependencies
        dependencies = Dependencies()
        outConfig = dependencies.reconfigureOrGetConfiguration(None)
        self.assertEqual(dependencies.config, outConfig)

    def test_reconfigureOrGetConfiguration_with_new_config_returns_it(self):
        from niprov.dependencies import Dependencies, Configuration
        dependencies = Dependencies()
        newSettings = Configuration()
        newSettings.verbose = True
        outConfig = dependencies.reconfigureOrGetConfiguration(newSettings)
        self.assertEqual(newSettings, outConfig)

    def test_reconfigureOrGetConfiguration_creates_new_dependencies(self):
        from niprov.dependencies import Dependencies, Configuration
        dependencies = Dependencies()
        self.assertEqual(dependencies.getListener().verbosity, 'info')
        newSettings = Configuration()
        newSettings.verbosity = 'warning'
        dependencies.reconfigureOrGetConfiguration(newSettings)
        self.assertEqual(dependencies.getListener().verbosity, 'warning')

    def test_GetConfiguration_returns_config(self):
        from niprov.dependencies import Dependencies
        dependencies = Dependencies()
        outConfig = dependencies.getConfiguration()
        self.assertEqual(dependencies.config, outConfig)

    def test_Filefactory_provided(self):
        from niprov.dependencies import Dependencies
        from niprov.files import FileFactory
        dependencies = Dependencies()
        self.assertIsInstance(dependencies.getFileFactory(), FileFactory)

    def test_provides_LocationFactory(self):
        from niprov.dependencies import Dependencies
        from niprov.locationfactory import LocationFactory
        dependencies = Dependencies()
        self.assertIsInstance(dependencies.getLocationFactory(), 
            LocationFactory)

    def test_provides_PipelineFactory(self):
        from niprov.dependencies import Dependencies
        from niprov.pipelinefactory import PipelineFactory
        dependencies = Dependencies()
        self.assertIsInstance(dependencies.getPipelineFactory(), 
            PipelineFactory)

    def test_provides_FormatFactory(self):
        from niprov.dependencies import Dependencies
        from niprov.formatfactory import FormatFactory
        dependencies = Dependencies()
        self.assertIsInstance(dependencies.getFormatFactory(), 
            FormatFactory)

    def test_provides_MediumFactory(self):
        from niprov.dependencies import Dependencies
        from niprov.mediumfactory import MediumFactory
        dependencies = Dependencies()
        self.assertIsInstance(dependencies.getMediumFactory(), 
            MediumFactory)

    def test_provides_Externals(self):
        from niprov.dependencies import Dependencies
        from niprov.externals import Externals
        dependencies = Dependencies()
        self.assertIsInstance(dependencies.getExternals(), 
            Externals)

    def test_provides_FileFilter(self):
        from niprov.dependencies import Dependencies
        from niprov.filefilter import FileFilter
        dependencies = Dependencies()
        self.assertIsInstance(dependencies.getFileFilter(), 
            FileFilter)

    def test_provides_Hasher(self):
        from niprov.dependencies import Dependencies
        from niprov.hashing import Hasher
        dependencies = Dependencies()
        self.assertIsInstance(dependencies.getHasher(), 
            Hasher)

    def test_provides_Users(self):
        from niprov.dependencies import Dependencies
        from niprov.users import Users
        dependencies = Dependencies()
        self.assertIsInstance(dependencies.getUsers(), 
            Users)

    def test_provides_Camera(self):
        from niprov.dependencies import Dependencies
        from niprov.camera import Camera
        dependencies = Dependencies()
        self.assertIsInstance(dependencies.getCamera(), 
            Camera)

    def test_provides_PictureCache(self):
        from niprov.dependencies import Dependencies
        from niprov.pictures import PictureCache
        dependencies = Dependencies()
        self.assertIsInstance(dependencies.getPictureCache(), 
            PictureCache)

    def test_provides_Query(self):
        from niprov.dependencies import Dependencies
        from niprov.querying import Query
        dependencies = Dependencies()
        self.assertIsInstance(dependencies.getQuery(), Query)




#!/usr/bin/python
# -*- coding: UTF-8 -*-
from mock import Mock, sentinel, patch
from tests.ditest import DependencyInjectionTestBase


class ViewerMediumTests(DependencyInjectionTestBase):

    def test_Returns_input(self):
        from niprov.mediumviewer import ViewerMedium
        exporter = ViewerMedium(sentinel.dependencies)
        with patch('niprov.mediumviewer.webbrowser') as webbrowser:
            exporter.export('the filename', sentinel.format)
            webbrowser.open.assert_called_with('the filename')

    def test_Does_nothing_if_passed_None(self):
        from niprov.mediumviewer import ViewerMedium
        exporter = ViewerMedium(sentinel.dependencies)
        with patch('niprov.mediumviewer.webbrowser') as webbrowser:
            exporter.export(None, sentinel.format)
            assert not webbrowser.open.called

    def test_Can_be_called_without_format(self):
        from niprov.mediumviewer import ViewerMedium
        exporter = ViewerMedium(sentinel.dependencies)
        with patch('niprov.mediumviewer.webbrowser') as webbrowser:
            exporter.export(None)






import unittest, os
from mock import Mock, patch, call
from tests.ditest import DependencyInjectionTestBase


class DiffTests(DependencyInjectionTestBase):

    def setUp(self):
        super(DiffTests, self).setUp()

    def test_areEqual_true_for_same_provenance(self):
        from niprov.diff import Diff
        diff = Diff(self.fileWithP({'a':1,'b':2}), 
                    self.fileWithP({'a':1,'b':2}))
        self.assertTrue(diff.areEqual())

    def test_areEqual_false_for_missing_key(self):
        from niprov.diff import Diff
        diff = Diff(self.fileWithP({'a':1,'b':2,'c':3}), 
                    self.fileWithP({'a':1,'b':2}))
        self.assertFalse(diff.areEqual())
        diff = Diff(self.fileWithP({'a':1,'b':2}), 
                    self.fileWithP({'a':1,'b':2,'d':4}))
        self.assertFalse(diff.areEqual())

    def test_areEqual_false_for_different_value(self):
        from niprov.diff import Diff
        diff = Diff(self.fileWithP({'a':1}), 
                    self.fileWithP({'a':2}))
        self.assertFalse(diff.areEqual())

    def test_can_ignore_key(self):
        from niprov.diff import Diff
        diff = Diff(self.fileWithP({'a':1}), 
                    self.fileWithP({'a':1,'b':3}))
        self.assertTrue(diff.areEqual(ignore=['b','c']))
        diff = Diff(self.fileWithP({'a':1,'b':3}), 
                    self.fileWithP({'a':2,'b':3}))
        self.assertTrue(diff.areEqual(ignore=['a','c']))

    def test_areEqual_ignores_id_by_default(self):
        from niprov.diff import Diff
        diff = Diff(self.fileWithP({'_id':1,'b':3}), 
                    self.fileWithP({'_id':2,'b':3}))
        self.assertTrue(diff.areEqual())
        diff = Diff(self.fileWithP({'_id':1,'b':3}), 
                    self.fileWithP({'b':3}))
        self.assertTrue(diff.areEqual())
        diff = Diff(self.fileWithP({'_id':1,'b':3}), 
                    self.fileWithP({'b':3}))
        self.assertTrue(diff.areEqual(ignore=['d']))

    def test_can_select_specific_keys_and_ignore_the_rest(self):
        from niprov.diff import Diff
        diff = Diff(self.fileWithP({'a':1,'b':2,'c':3}), 
                    self.fileWithP({'a':1,'b':9,'c':3,'d':4}))
        self.assertTrue(diff.areEqual(select=['a','c']))
        self.assertFalse(diff.areEqual(select=['b']))
        self.assertFalse(diff.areEqual(select=['d']))

    def test_assertEqual_throws_for_different_value(self):
        from niprov.diff import Diff
        diff = Diff(self.fileWithP({'a':1}), 
                    self.fileWithP({'a':2}))
        with self.assertRaises(AssertionError):
            diff.assertEqual()

    def test_areEqualProtocol_selects_on_BaseFile_getProtocolFields(self):
        from niprov.diff import Diff
        diff = Diff(self.fileWithP({'a':1,'x':7}, protocol=['x']), 
                    self.fileWithP({'a':2,'x':7}))
        self.assertTrue(diff.areEqualProtocol())
        diff = Diff(self.fileWithP({'a':1,'x':7}, protocol=['x']), 
                    self.fileWithP({'a':2,'x':8}))
        self.assertFalse(diff.areEqualProtocol())

    def test_assertEqualProtocol_throws_if_not_areEqualProtocol(self):
        from niprov.diff import Diff
        diff = Diff(self.fileWithP({'a':1,'x':7}, protocol=['x']), 
                    self.fileWithP({'a':2,'x':8}))
        with self.assertRaises(AssertionError):
            diff.assertEqualProtocol()

    def test_getDifferenceString(self):
        from niprov.diff import Diff
        n = Diff.NCHARSCOL
        diff = Diff(self.fileWithP({'a':1}), 
                    self.fileWithP({'a':2,'b':3}))
        diffStr = diff.getDifferenceString()
        line = ' '.ljust(n)+'afilename'.ljust(n)+' '+'afilename'.ljust(n)
        self.assertIn(line, diffStr)
        line = 'a'.ljust(n)+' '+str(1).ljust(n)+' '+str(2).ljust(n)
        self.assertIn(line, diffStr)
        line = 'b'.ljust(n)+' '+'n/a'.ljust(n)+' '+str(3).ljust(n)
        self.assertIn(line, diffStr)

    def test_assertEqual_exception_message_is_getDifferenceString(self):
        from niprov.diff import Diff
        n = Diff.NCHARSCOL
        diff = Diff(self.fileWithP({'a':1}), 
                    self.fileWithP({'a':2,'b':2}))
        exception = None
        try:
            diff.assertEqual()
        except AssertionError as e:
            exception = e
        self.assertIsNotNone(exception, 'No AssertionError raised')
        self.assertEqual(diff.getDifferenceString(), str(e))

    def test_Diff_to_string_is_(self):
        from niprov.diff import Diff
        diff = Diff(self.fileWithP({'a':1}), 
                    self.fileWithP({'a':2}))
        self.assertEqual(diff.getDifferenceString(), str(diff))

    def test_getSame(self):
        from niprov.diff import Diff
        diff = Diff(self.fileWithP({'a':1,'b':2}), 
                    self.fileWithP({'a':1,'b':3}))
        self.assertEqual(diff.getSame(), {'a':'same'})

    def test_getSameString(self):
        from niprov.diff import Diff
        n = Diff.NCHARSCOL
        diff = Diff(self.fileWithP({'a':1}), 
                    self.fileWithP({'a':1,'b':3}))
        diffStr = diff.getSameString()
        line = ' '.ljust(n)+'afilename'.ljust(n)+' '+'afilename'.ljust(n)
        self.assertIn(line, diffStr)
        line = 'a'.ljust(n)+' '+str(1).ljust(n)+' '+str(1).ljust(n)
        self.assertIn(line, diffStr)

    def fileWithP(self, provenance, protocol=None):
        mfile = Mock()
        mfile.location = 'afilename'
        mfile.getProvenance.return_value = provenance
        mfile.getProtocolFields.return_value = protocol
        return mfile






import getpass

class Users(object):

    def __init__(self, dependencies):
        self.config = dependencies.getConfiguration()

    def determineUser(self, passedUserValue):
        """Determine the name of the user creating provenance.

        The following methods will be tried to determine the username;
        1) The passedUserValue argument if not None
        2) The configuration file or setting has a value for the key user
        3) The OS username.

        Args:
            passedUserValue (str): This value will override any other method
                to determine the username.

        Returns:
            string: The username or handle of the user.
        """
        if passedUserValue is not None:
            return passedUserValue
        if self.config.user:
            return self.config.user
        else:
            return getpass.getuser()



from pyramid.view import view_config
import os
import niprov.searching as searching
from pyramid.httpexceptions import HTTPNotFound


@view_config(route_name='home', renderer='templates/home.mako')
def home(request):
    return {}

@view_config(route_name='latest', renderer='templates/list.mako')
def latest(request):
    repository = request.dependencies.getRepository()
    return {'images':repository.latest()}

@view_config(route_name='short', renderer='templates/single.mako')
def short(request):
    sid = request.matchdict['id']
    repository = request.dependencies.getRepository()
    query = request.dependencies.getQuery()
    image = repository.byId(sid)
    if not image:
        raise HTTPNotFound
    return {'image': image, 'copies': query.copiesOf(image)}

@view_config(route_name='location', renderer='templates/single.mako')
def location(request):
    path = os.sep + os.path.join(*request.matchdict['path'])
    loc = request.matchdict['host'] + ':' + path
    query = request.dependencies.getQuery()
    image = query.byLocation(loc)
    if not image:
        raise HTTPNotFound
    return {'image': image, 'copies': query.copiesOf(image)}

@view_config(route_name='stats', renderer='templates/stats.mako')
def stats(request):
    repository = request.dependencies.getRepository()
    return {'stats':repository.statistics()}

@view_config(route_name='pipeline', renderer='templates/pipeline.mako')
def pipeline(request):
    sid = request.matchdict['id']
    files = request.dependencies.getRepository()
    pipeline = request.dependencies.getPipelineFactory()
    targetFile = files.byId(sid)
    if not targetFile:
        raise HTTPNotFound
    return {'pipeline':pipeline.forFile(targetFile), 'sid':sid}

@view_config(route_name='subject', renderer='templates/list.mako')
def subject(request):
    subj = request.matchdict['subject']
    query = request.dependencies.getQuery()
    return {'images':query.bySubject(subj)}

@view_config(route_name='project', renderer='templates/list.mako')
def project(request):
    project = request.matchdict['project']
    query = request.dependencies.getQuery()
    return {'images':query.byProject(project)}

@view_config(route_name='user', renderer='templates/list.mako')
def user(request):
    user = request.matchdict['user']
    query = request.dependencies.getQuery()
    return {'images':query.byUser(user)}

@view_config(route_name='modality', renderer='templates/list.mako')
def modality(request):
    modality = request.matchdict['modality']
    query = request.dependencies.getQuery()
    return {'images':query.byModality(modality)}

@view_config(route_name='search', renderer='templates/list.mako')
def search(request):
    text = request.GET['text']
    results = searching.search(text, request.dependencies)
    return {'images':results, 'searchtext':text}

@view_config(route_name='modalities', renderer='templates/category.mako')
def modalities(request):
    query = request.dependencies.getQuery()
    return {'category':'modality', 'categoryPlural':'modalities', 
            'items':query.allModalities()}

@view_config(route_name='projects', renderer='templates/category.mako')
def projects(request):
    query = request.dependencies.getQuery()
    return {'category':'project', 'categoryPlural':'projects', 
            'items':query.allProjects()}

@view_config(route_name='users', renderer='templates/category.mako')
def users(request):
    query = request.dependencies.getQuery()
    return {'category':'user', 'categoryPlural':'users', 
            'items':query.allUsers()}





from tests.ditest import DependencyInjectionTestBase
from mock import Mock, sentinel
from datetime import datetime

class BaseFileTests(DependencyInjectionTestBase):

    def setUp(self):
        super(BaseFileTests, self).setUp()
        self.path = 'example.abc'
        self.location.toDictionary.return_value = {'path':self.path}
        from niprov.basefile import BaseFile
        self.constructor = BaseFile
        self.file = BaseFile(self.path, dependencies=self.dependencies)

    def test_Saves_file_path_along_with_provenance(self):
        out = self.file.inspect()
        self.assertEqual(out['path'], self.path)

    def test_Saves_filesize_along_with_provenance(self):
        out = self.file.inspect()
        self.assertEqual(out['size'], self.filesys.getsize(self.path))

    def test_Saves_file_creation_time_along_with_provenance(self):
        out = self.file.inspect()
        self.assertEqual(out['created'], self.filesys.getctime(self.path))

    def test_Asks_hasher_for_digest_of_file(self):
        out = self.file.inspect()
        self.assertEqual(out['hash'], self.hasher.digest(self.path))

    def test_Provenance_property_equals_dictionary_returned_by_inspect(self):
        out = self.file.inspect()
        self.assertEqual(out, self.file.provenance)

    def test_Attach_method(self):
        pass

    def test_Series_interface(self):
        self.assertEqual(self.file.getSeriesId(), None)

    def test_Construct_with_provenance(self):
        prov = {'aprop':'aval'}
        img = self.constructor(self.path, provenance=prov, dependencies=self.dependencies)
        self.assertEqual(prov, img.provenance)

    def test_Construct_with_provenance_adds_path_if_not_set(self):
        prov = {'aprop':'aval'}
        img = self.constructor(self.path, provenance=prov, dependencies=self.dependencies)
        self.assertIn('path', img.provenance)
        self.assertEqual(self.path, img.provenance['path'])

    def test_Inspect_leaves_existing_fields_updates_others(self):
        prov = {'aprop':'aval','size':9876}
        img = self.constructor(self.path, provenance=prov, 
            dependencies=self.dependencies)
        img.inspect()
        self.assertEqual(img.provenance['aprop'], 'aval')
        self.assertEqual(img.provenance['size'], self.filesys.getsize(self.path))

    def test_Uses_location_objects_to_add_location_info_to_provenance(self):
        self.location.toDictionary.return_value = {'newLocKey':'newLocValue',
            'path':'newPathVal'}
        img = self.constructor(self.path, dependencies=self.dependencies)
        self.locationFactory.fromString.assert_called_with(self.path)
        self.assertEqual('newLocValue', img.provenance['newLocKey'])
        self.assertEqual('newPathVal', img.provenance['path'])
        self.assertEqual(self.location, img.location)

    def test_parents_property(self):
        prov = {'aprop':'aval','size':9876}
        img = self.constructor(self.path, provenance=prov, 
            dependencies=self.dependencies)
        self.assertEqual(img.parents, [])
        prov = {'aprop':'aval','parents':['foo','bar']}
        img2 = self.constructor(self.path, provenance=prov, 
            dependencies=self.dependencies)
        self.assertEqual(img2.parents, ['foo','bar'])

    def test_getProvenance_provides_prov_serialized_in_requested_format(self):
        img = self.constructor(self.path, dependencies=self.dependencies)
        out = img.getProvenance('myformat')
        self.formatFactory.create.assert_called_with('myformat')
        self.format.serialize.assert_called_with(img)
        self.assertEqual(self.format.serialize(), out)

    def test_viewSnapshot_uses_ViewerMedium_and_PictureCache(self):
        viewer = Mock()
        self.mediumFactory.create.return_value = viewer
        self.pictureCache.getFilepath.return_value = 'snapshot filepath'
        img = self.constructor(self.path, dependencies=self.dependencies)
        img.viewSnapshot()
        self.pictureCache.getFilepath.assert_called_with(for_=img)
        viewer.export.assert_called_with('snapshot filepath')

    def test_getSnapshotFilepath(self):
        self.pictureCache.getFilepath.return_value = 'snapshot filepath'
        img = self.constructor(self.path, dependencies=self.dependencies)
        self.assertEqual(img.getSnapshotFilepath(), 'snapshot filepath')

    def test_Determines_modality(self):
        img = self.constructor(self.path, dependencies=self.dependencies)
        img.inspect()
        self.assertEqual(img.provenance['modality'], 'other')

    def test_Preserves_modality_if_inherited(self):
        img = self.constructor(self.path, dependencies=self.dependencies,
                                provenance={'modality':'magic'})
        img.inspect()
        self.assertEqual(img.provenance['modality'], 'magic')

    def test_On_construction_has_status_new(self):
        img = self.constructor(self.path, dependencies=self.dependencies)
        self.assertEqual(img.status, 'new')

    def test_keepVersionsFromPrevious(self):
        img = self.constructor(self.path, dependencies=self.dependencies)
        prev = Mock()
        prev.provenance = {'y':1501, '_versions':[{'y':1499},{'y':1500}]}
        img.keepVersionsFromPrevious(prev)
        self.assertEqual('new-version', img.status)
        self.assertEqual(3, len(img.provenance['_versions']))
        self.assertEqual({'y':1501}, img.provenance['_versions'][-1])
        self.assertEqual({'y':1500}, img.provenance['_versions'][-2])
        self.assertEqual({'y':1499}, img.provenance['_versions'][-3])

    def test_versions_property_access(self):
        img = self.constructor(self.path, dependencies=self.dependencies)
        img.provenance = {'_versions':sentinel.versions}
        self.assertEqual(img.provenance['_versions'], img.versions)



import unittest, os, shutil, sys
from cStringIO import StringIO 
absp = os.path.abspath


class TerminalApiTests(unittest.TestCase):

    def setUp(self):
        self.dbpath = os.path.expanduser(os.path.join('~','provenance_test.json'))
        if os.path.exists(self.dbpath):
            os.remove(self.dbpath)
        os.mkdir('temp')
        from niprov import ProvenanceContext
        self.provenance = ProvenanceContext()
        self.provenance.config.database_type = 'file'
        self.provenance.config.database_url = self.dbpath
        self.provenance.config.verbose = False
        self.oldstdout = sys.stdout
        sys.stdout = self.stdout = StringIO()

    def tearDown(self):
        sys.stdout = self.oldstdout
        self.stdout.close()
        shutil.rmtree('temp')

    @unittest.skip("Doesn't work on travis")
    def test_Report_pipeline(self):
        self.provenance.discover('testdata')
        raw = os.path.abspath('testdata/eeg/stub.cnt')
        self.provenance.log(absp('p1a.f'), 'test', raw, transient=True)
        self.provenance.log(absp('p1b.f'), 'test', raw, transient=True)
        self.provenance.log(absp('p2.f'), 'test', absp('p1a.f'), transient=True)
        self.provenance.export(None, 'stdout', 'simple', pipeline=True)
        exp = ''
        exp += '+---stub.cnt\n'
        exp += '|   +---p1a.f\n'
        exp += '|   |   +---p2.f\n'
        #exp += '|   +---p1b.f\n' # p1b is not related to p2f
        import time
        time.sleep(0.3)
        self.assertIn(exp, self.stdout.getvalue())


if __name__ == '__main__':
    unittest.main()




from xml.dom.minidom import parse

def getNonTextChildNodes(node):
    return [n for n in node.childNodes if n.nodeType != n.TEXT_NODE]

def getElementsByTagNameWithoutPrefix(node, partialTagName):
    childNodes = getNonTextChildNodes(node)
    tagLength = len(partialTagName)
    matchingNodes = []
    for childNode in childNodes:
        if len(childNode.tagName) >= tagLength:
            if childNode.tagName[-tagLength:] == partialTagName:
                matchingNodes.append(childNode)
    return matchingNodes

def getTextContent(node):
    textChildNodes = [n for n in node.childNodes if n.nodeType == n.TEXT_NODE]
    return ''.join([n.data for n in textChildNodes])

def getTextContentForFirstElementByTag(node, tagname):
    firstChildByTag = node.getElementsByTagName(tagname)[0]
    return getTextContent(firstChildByTag)

def getRefForFirstElementByTag(node, tagname):
    firstChildByTag = node.getElementsByTagName(tagname)[0]
    href = firstChildByTag.getAttribute('href')
    return href[1:]

def getElementById(node, targetId):
    for childNode in node.childNodes:
        if childNode.nodeType != childNode.TEXT_NODE:
            if childNode.hasAttribute('id'):
                idval = childNode.getAttribute('id')
                if idval == targetId:
                    return childNode
    raise ValueError('Cannot find element with id')



from collections import namedtuple
QueryField = namedtuple('QueryField', ['name', 'value', 'all'])

class Query(object):

    def __init__(self, dependencies):
        self.fields = []
        self.repository = dependencies.getRepository()
        self.location = dependencies.getLocationFactory()
        self.cachedResults = None

    def __iter__(self):
        if self.cachedResults is None:
            self.cachedResults = self.repository.inquire(self)
        return self.cachedResults.__iter__()

    def __len__(self):
        if self.cachedResults is None:
            self.cachedResults = self.repository.inquire(self)
        return len(self.cachedResults)

    def __contains__(self, key):
        if self.cachedResults is None:
            self.cachedResults = self.repository.inquire(self)
        return key in self.cachedResults

    def _fieldHasValue(self, field, value):
        return QueryField(field, value=value, all=False)

    def _fieldAllValues(self, field):
        return QueryField(field, value=None, all=True)

    def getFields(self):
        return self.fields

    def byLocation(self, val):
        val = self.location.completeString(val)
        return self.repository.byLocation(val)

    def all(self):
        return self.repository.all()

    def latest(self):
        return self.repository.latest()

    def statistics(self):
        return self.repository.statistics()

    def byModality(self, val):
        self.fields.append(self._fieldHasValue('modality', val))
        return self

    def byProject(self, val):
        self.fields.append(self._fieldHasValue('project', val))
        return self

    def byUser(self, val):
        self.fields.append(self._fieldHasValue('user', val))
        return self

    def bySubject(self, val):
        self.fields.append(self._fieldHasValue('subject', val))
        return self

    def byApproval(self, val):
        self.fields.append(self._fieldHasValue('approval', val))
        return self

    def allModalities(self):
        self.fields.append(self._fieldAllValues('modality'))
        return self

    def allProjects(self):
        self.fields.append(self._fieldAllValues('project'))
        return self

    def allUsers(self):
        self.fields.append(self._fieldAllValues('user'))
        return self

    def copiesOf(self, target):
        checksum = target.provenance.get('hash', None)
        filesize = target.provenance.get('size', 0)
        if checksum and filesize > 0:
            self.fields.append(self._fieldHasValue('hash', checksum))
        else:
            self.cachedResults = []
        return self



# -*- coding: utf-8 -*-
"""mnefun support module

This module provides handlers to attach to mnefun events.

::

    import mnefun
    import niprov.mnefunsupport
    params = mnefun.Params()
    params.on_process = niprov.mnefunsupport.handler

"""
import os
from niprov.dependencies import Dependencies
from niprov import ProvenanceContext


def handler(text, func, out, params, dependencies=Dependencies()):
    """mnefun on_process handler

    Responds to the following mnefun steps:
    fetch_raw_files: Runs discover on the subjects' raw data dirs.
    fetch_sss_files,
    apply_preprocessing_combined,
    save_epochs: For these steps, runs log for the new files.
    """
    listener = dependencies.getListener()
    libs = dependencies.getLibraries()
    provenance = ProvenanceContext()
    funcname = func.func_name
    listener.mnefunEventReceived(funcname)
    paramdict = {}
    for paramname in dir(params):
        objectvalue = getattr(params, paramname)
        blacklist = ['get_projs_from', 'inv_runs']
        if hasattr(objectvalue, '__call__'):
            continue
        if paramname[0] == '_':
            continue
        if 'array' in str(type(objectvalue)):
            continue
        if paramname in blacklist:
            continue
        paramdict[paramname] = objectvalue
    subjects = [params.subjects[i] for i in params.subject_indices]
    for subj in subjects:
        customprov = {'mnefun':paramdict}
        if funcname in ['fetch_raw_files', 'score_fun', 'fetch_sss_files']:
            rawfiles = libs.mnefun.get_raw_fnames(params, subj, 'raw', add_splits=True,
                                                  run_indices=params.subject_run_indices)
        if funcname == 'fetch_raw_files':
            for f in rawfiles:
                provenance.add(f, provenance=customprov)
        elif funcname == 'score_fun':
            eventfiles = libs.mnefun._paths.get_event_fnames(params, subj,
                                                             params.subject_run_indices)
            for rawfile, eventfile in zip(rawfiles, eventfiles):
                provenance.log(eventfile, 'scoring', rawfile, provenance=customprov)
        elif funcname == 'fetch_sss_files':
            trans = 'Signal Space Separation'
            sssfiles = libs.mnefun.get_raw_fnames(params, subj, 'sss')
            for rawfile, sssfile in zip(rawfiles, sssfiles):
                provenance.log(sssfile, trans, rawfile, provenance=customprov)
        elif funcname == 'apply_preprocessing_combined':
            trans = 'Signal Space Projection'
            sssfiles = libs.mnefun.get_raw_fnames(params, subj, 'sss')
            pcafiles = libs.mnefun.get_raw_fnames(params, subj, 'pca')
            for sssfile, pcafile in zip(sssfiles, pcafiles):
                provenance.log(pcafile, trans, sssfile, provenance=customprov)
        elif funcname == 'save_epochs':
            pcafiles = libs.mnefun.get_raw_fnames(params, subj, 'pca')
            evtfiles = libs.mnefun._paths.get_epochs_evokeds_fnames(params, 
                subj, params.analyses)
            for evtfile in evtfiles:
                if os.path.isfile(evtfile[0]):
                    provenance.log(evtfile[0], 'Epoching', pcafiles, provenance=customprov)


#        self.reject = dict(eog=np.inf, grad=1500e-13, mag=5000e-15, eeg=150e-6)
#        self.flat = dict(eog=-1, grad=1e-13, mag=1e-15, eeg=1e-6)



import pymongo, copy, bson
from niprov.dependencies import Dependencies


class MongoRepository(object):

    def __init__(self, dependencies=Dependencies()):
        self.config = dependencies.getConfiguration()
        self.factory = dependencies.getFileFactory()
        self.pictures = dependencies.getPictureCache()
        client = pymongo.MongoClient(self.config.database_url)
        self.db = client.get_default_database()

    def byLocation(self, locationString):
        """Get the provenance for a file at the given location. 

        In the case of a dicom series, this returns the provenance for the 
        series.

        Args:
            locationString (str): Location of the image file.

        Returns:
            dict: Provenance for one image file.
        """
        record = self.db.provenance.find_one({'location':locationString})
        return self.inflate(record)

    def byLocations(self, listOfLocations):
        records = self.db.provenance.find({'location':{'$in':listOfLocations}})
        return [self.inflate(record) for record in records]

    def getSeries(self, image):
        """Get the object that carries provenance for the series that the image 
        passed is in. 

        Args:
            image (:class:`.DicomFile`): File that is part of a series.

        Returns:
            :class:`.DicomFile`: Image object that caries provenance for 
                         the series.
        """
        seriesUid = image.getSeriesId()
        if seriesUid is None:
            return None
        record = self.db.provenance.find_one({'seriesuid':seriesUid})
        return self.inflate(record)

    def add(self, image):
        """Add the provenance for one file to storage.

        Args:
            image (:class:`.BaseFile`): Image file to store.
        """
        self.db.provenance.insert_one(self.deflate(image))

    def update(self, image):
        """Save changed provenance for this file..

        Args:
            image (:class:`.BaseFile`): Image file that has changed.
        """
        self.db.provenance.update({'location':image.location.toString()}, 
            self.deflate(image))

    def updateApproval(self, locationString, approvalStatus):
        self.db.provenance.update({'location':locationString}, 
            {'$set': {'approval': approvalStatus}})

    def all(self):
        """Retrieve all known provenance from storage.

        Returns:
            list: List of provenance for known files.
        """
        records = self.db.provenance.find()
        return [self.inflate(record) for record in records]

    def latest(self):
        records = self.db.provenance.find().sort('added', -1).limit(20)
        return [self.inflate(record) for record in records]

    def statistics(self):
        grps = self.db.provenance.aggregate(
           [{'$group':
                 {
                   '_id': None,
                   'totalsize': { '$sum': '$size' },
                   'count': { '$sum': 1 }
                 }
            }])
        groups = list(grps)
        if len(list(groups)) == 0:
            return {'count':0}
        return list(groups)[0]

    def byId(self, uid):
        record = self.db.provenance.find_one({'id':uid})
        return self.inflate(record)

    def byParents(self, listOfParentLocations):
        records = self.db.provenance.find({'parents':{
            '$in':listOfParentLocations}})
        return [self.inflate(record) for record in records]

    def inquire(self, query):
        field = query.getFields()[0]
        if field.all:
            records = self.db.provenance.distinct(field.name)
            return records
        else:
            records = self.db.provenance.find({field.name:field.value})
            return [self.inflate(record) for record in records]

    def search(self, text):
        searchfields = ['location','user','subject','project','protocol',
                  'transformation','technique','modality']
        indexspec = [(field, pymongo.TEXT) for field in searchfields]
        self.db.provenance.create_index(indexspec, name='textsearch')
        records = self.db.provenance.find({'$text':{'$search': text}})
        return [self.inflate(record) for record in records]

    def deflate(self, img):
        record = copy.deepcopy(img.provenance)
        snapshotData = self.pictures.getBytes(for_=img)
        if snapshotData:
            record['_snapshot-data'] = bson.Binary(snapshotData)
        return record

    def inflate(self, record):
        if record is None:
            return None
        img = self.factory.fromProvenance(record)
        if '_snapshot-data' in record:
            self.pictures.keep(record['_snapshot-data'], for_=img)
        return img




import unittest
from mock import sentinel, Mock
from tests.ditest import DependencyInjectionTestBase

class ExportingTests(DependencyInjectionTestBase):

    def setUp(self):
        super(ExportingTests, self).setUp()

    def export(self, *args, **kwargs):
        import niprov
        return niprov.export(*args, dependencies=self.dependencies, **kwargs)

    def test_Obtains_format_from_factory(self):
        self.export(None, form='json', medium='x')
        self.formatFactory.create.assert_any_call('json')
        self.export(None, form='xml', medium='x')
        self.formatFactory.create.assert_any_call('xml')

    def test_Obtains_medium_from_factory(self):
        self.export(None, medium='file', form='x')
        self.mediumFactory.create.assert_any_call('file')
        self.export(None, medium='stdout', form='x')
        self.mediumFactory.create.assert_any_call('stdout')

    def test_Passes_provenance_through_format_and_medium(self):
        self.format.serialize.return_value = 'serialized prov'
        self.medium.export.return_value = sentinel.mediumOutput
        out = self.export(sentinel.images, 'a medium','a format')
        self.format.serialize.assert_any_call(sentinel.images)
        self.medium.export.assert_called_with('serialized prov', self.format)
        self.assertEqual(sentinel.mediumOutput, out)

    def test_Passes_single_item_of_provenance_to_exporter(self):
        self.export(sentinel.image, 'a medium','a format')
        self.format.serialize.assert_any_call(sentinel.image)

    def test_Can_export_pipeline_for_file(self):
        out = self.export(sentinel.image, 'a medium','a format', pipeline=True)
        self.pipelineFactory.forFile.assert_called_with(sentinel.image)
        self.format.serialize.assert_called_with(
            self.pipelineFactory.forFile())

    def test_Backup_passes_all_images_to_export(self):
        import niprov.exporting
        niprov.exporting.export = Mock()
        niprov.exporting.backup(self.dependencies)
        niprov.exporting.export.assert_called_with(self.query.all(), 
            medium='file', form='json', dependencies=self.dependencies)







import unittest, os
from mock import Mock, patch, call, sentinel
from tests.ditest import DependencyInjectionTestBase


class AddTests(DependencyInjectionTestBase):

    def setUp(self):
        super(AddTests, self).setUp()
        self.config.dryrun = False
        self.repo.byLocation.return_value = None
        self.query.copiesOf.return_value = []
        self.img = Mock()
        self.lastProvenance = None
        def locAt(loc, provenance):
            self.lastProvenance = provenance
            self.lastPath = loc
            return self.img
        self.fileFactory.locatedAt.side_effect = locAt
        patcher = patch('niprov.adding.datetime')
        self.datetime = patcher.start()
        self.addCleanup(patcher.stop)

    def add(self, path, **kwargs):
        from niprov.adding import add
        with patch('niprov.adding.inheritFrom') as self.inheritFrom:
            return add(path, dependencies=self.dependencies, **kwargs)

    def assertNotCalledWith(self, m, *args, **kwargs):
        c = call(*args, **kwargs)
        assert c not in m.call_args_list, "Unexpectedly found call: "+str(c)

    def test_Returns_provenance_and_informs_listener(self):
        new = '/p/f2'
        image = self.add(new)
        self.listener.fileAdded.assert_called_with(self.img)
        self.assertEqual(image, self.img)

    def test_Sets_transient_flag_if_provided(self):
        image = self.add('/p/f1', transient=True)
        self.assertEqual(self.lastProvenance['transient'],True)

    def test_Creates_ImageFile_object_with_factory(self):
        image = self.add('p/afile.f')
        self.assertIs(self.img, image)

    def test_Calls_inspect(self):
        image = self.add('p/afile.f')
        self.img.inspect.assert_called_with()

    def test_If_inspect_raises_exceptions_tells_listener_and_doesnt_save(self):
        self.img.inspect.side_effect = IOError
        image = self.add('p/afile.f')
        assert not self.repo.add.called
        assert not self.repo.update.called
        self.listener.fileError.assert_called_with(self.img.path)
        self.assertEqual(self.img.status, 'failed')

    def test_If_dryrun_doesnt_talk_to_repo_and_status_is_test(self):
        self.config.dryrun = True
        image = self.add('p/afile.f')
        assert not self.repo.add.called
        assert not self.repo.update.called
        assert not self.img.inspect.called

    def test_accepts_optional_provenance(self):
        image = self.add('p/afile.f', provenance={'fob':'bez'})
        self.assertEqual(self.lastProvenance['fob'],'bez')

    def test_If_file_doesnt_exists_raises_error(self):
        self.filesys.fileExists.return_value = False
        self.assertRaises(IOError, self.add, self.img.location.path)
        self.filesys.fileExists.assert_called_with(self.img.location.path)

    def test_For_nonexisting_transient_file_behaves_normal(self):
        self.filesys.fileExists.return_value = False
        self.add('p/afile.f', transient=True)

    def test_Doesnt_inspect_transient_files(self):
        self.add('p/afile.f', transient=True)
        assert not self.img.inspect.called

    def test_Adds_timestamp(self):
        image = self.add('p/afile.f')
        self.assertEqual(self.lastProvenance['added'],self.datetime.now())

    def test_Adds_uid(self):
        with patch('niprov.adding.shortuuid') as shortuuid:
            shortuuid.uuid.return_value = 'abcdefghijklmn'
            image = self.add('p/afile.f')
            self.assertEqual(self.lastProvenance['id'],'abcdef')

    def test_If_config_attach_set_calls_attach_on_file(self):
        self.config.attach = False
        self.add('p/afile.f')
        assert not self.img.attach.called, "Shouldnt attach if not configured."
        self.config.attach = True
        self.config.attach_format = 'abracadabra'
        self.add('p/afile.f', transient=True)
        assert not self.img.attach.called, "Shouldnt attach to transient file."
        self.add('p/afile.f')
        self.img.attach.assert_called_with('abracadabra')

    def test_If_file_unknown_adds_it(self):                             # A
        self.repo.byLocation.return_value = None
        self.repo.getSeries.return_value = None
        image = self.add('p/afile.f')
        self.repo.add.assert_any_call(self.img)

    def test_If_file_is_version_but_not_series(self):                    # B
        previousVersion = Mock()
        self.repo.byLocation.return_value = previousVersion
        self.repo.getSeries.return_value = None
        img = self.add('p/afile.f')
        self.img.keepVersionsFromPrevious.assert_called_with(previousVersion)
        self.repo.update.assert_any_call(self.img)

    def test_If_file_is_version_and_series(self):                        # C
        previousVersion = Mock()
        series = Mock()
        self.repo.byLocation.return_value = previousVersion
        self.repo.getSeries.return_value = series
        image = self.add('p/afile.f')
        self.img.keepVersionsFromPrevious.assert_called_with(previousVersion)
        self.repo.update.assert_any_call(self.img)

    def test_If_file_not_version_but_series_and_not_in_there_yet(self):   # D1
        series = Mock()
        series.hasFile.return_value = False
        series.mergeWith.return_value = series
        self.repo.byLocation.return_value = None
        self.repo.getSeries.return_value = series
        image = self.add('p/afile.f')
        series.mergeWith.assert_called_with(self.img)
        self.repo.update.assert_any_call(series)

    def test_If_file_not_version_but_series_has_file(self):               # D2
        series = Mock()
        series.hasFile.return_value = True
        self.repo.byLocation.return_value = None
        self.repo.getSeries.return_value = series
        image = self.add('p/afile.f')
        assert not series.mergeWith.called
        self.img.keepVersionsFromPrevious.assert_called_with(series)
        self.repo.update.assert_any_call(self.img)

    def test_copiesOf_not_called_before_inspect(self):
        def testIfInspectedAndReturnEptyList(img):
            img.inspect.assert_called_with()
            return []
        self.query.copiesOf.side_effect = testIfInspectedAndReturnEptyList
        image = self.add('p/afile.f')

    def test_getSeries_not_called_before_inspect(self):
        self.repo.getSeries.side_effect = lambda img: img.inspect.assert_called_with()
        image = self.add('p/afile.f')

    def test_copiesOf_not_called_if_parent_available(self):
        image = self.add('p/afile.f', provenance={'parents':[sentinel.parent]})
        assert not self.query.copiesOf.called

    def test_Found_copy_set_as_parent_inherits_and_flags_and_informs_listener(self):
        self.img.provenance = {}
        copy = Mock()
        copy.provenance = {'location':'copy-location'}
        self.query.copiesOf.return_value = [self.img, copy]
        out = self.add('p/afile.f')
        self.inheritFrom.assert_called_with(self.img.provenance, copy.provenance)
        self.listener.usingCopyAsParent.assert_called_with(copy)
        self.assertEqual(copy.location.toString(), out.provenance['parents'][0])
        self.assertEqual(True, out.provenance['copy-as-parent'])

    def test_If_only_copy_is_same_location_ignores_it(self):
        self.img.provenance = {}
        self.query.copiesOf.return_value = [self.img]
        out = self.add('p/afile.f')
        assert not self.inheritFrom.called
        assert not self.listener.usingCopyAsParent.called
        self.assertNotIn('parents', out.provenance)
        self.assertNotIn('copy-as-parent', out.provenance)

        




from niprov.discovery import discover
from niprov.inspection import inspect
from niprov.logging import log
from niprov.recording import record
from niprov.adding import add
from niprov.renaming import renameDicoms
from niprov.approval import (markForApproval, markedForApproval, approve, 
    selectApproved)
from niprov.context import ProvenanceContext
from niprov.config import Configuration
from niprov.webapp import serve
from niprov.exporting import export, print_, backup, view
from niprov.importing import importp
from niprov.comparing import compare
from niprov.searching import search





import os
from mock import Mock, patch, call, sentinel
from tests.ditest import DependencyInjectionTestBase


class PipelineFactoryTests(DependencyInjectionTestBase):

    def setUp(self):
        super(PipelineFactoryTests, self).setUp()

#    def test_forFile_returns_Pipeline(self):
#        from niprov.pipelinefactory import PipelineFactory
#        from niprov.pipeline import Pipeline
#        self.repo.byLocations.return_value = []
#        factory = PipelineFactory(dependencies=self.dependencies)
#        target = Mock()
#        target.provenance = {}
#        pipeline = factory.forFile(target)
#        self.assertIsInstance(pipeline, Pipeline)

    def test_forFile_makes_pipeline_with_targets_parents_parents(self):
        from niprov.pipelinefactory import PipelineFactory
        p1a = self.fileWithLocation('p1a')
        p1b = self.fileWithLocation('p1b')
        p2a = self.fileWithLocation('p2a')
        p2b = self.fileWithLocation('p2b')
        p1a.provenance['parents'] = ['p2a']
        p1b.provenance['parents'] = ['p2a','p2b']
        t = self.fileWithLocation('t')
        t.provenance['parents'] = ['p1a','p1b']
        repodict = {'t':t,'p1a':p1a, 'p1b':p1b, 'p2a':p2a, 'p2b':p2b}
        self.repo.byLocations.side_effect = lambda ls: [repodict[l] for l in ls]
        self.repo.byParents.side_effect = lambda ls: []
        factory = PipelineFactory(dependencies=self.dependencies)
        with patch('niprov.pipelinefactory.Pipeline') as PipelineCtr:
            pipeline = factory.forFile(t)
            PipelineCtr.assert_called_with(repodict.values())
            self.assertEqual(3, self.repo.byLocations.call_count)
            self.assertEqual(1, self.repo.byParents.call_count)

    def test_forFile_makes_pipeline_with_targets_children(self):
        from niprov.pipelinefactory import PipelineFactory
        c1a = self.fileWithLocation('c1a')
        c1b = self.fileWithLocation('c1b')
        c2a = self.fileWithLocation('c2a')
        c2b = self.fileWithLocation('c2b')
        c1a.provenance['parents'] = ['t']
        c1b.provenance['parents'] = ['t']
        c2a.provenance['parents'] = ['c1a']
        c2b.provenance['parents'] = ['c1a','c1b']
        t = self.fileWithLocation('t')
        repodict = {'t':t,'c1a':c1a, 'c1b':c1b, 'c2a':c2a, 'c2b':c2b}
        childrenByParent = {'t':[c1a, c1b],'c1a':[c2a],'c1b':[c2a, c2b],
            'c2a':[],'c2b':[]}
        self.repo.byLocations.side_effect = lambda ls: []
        self.repo.byParents.side_effect = lambda ps: [c for p in ps for c in childrenByParent[p]]
        factory = PipelineFactory(dependencies=self.dependencies)
        with patch('niprov.pipelinefactory.Pipeline') as PipelineCtr:
            pipeline = factory.forFile(t)
            PipelineCtr.assert_called_with(repodict.values())
            self.assertEqual(1, self.repo.byLocations.call_count)
            self.assertEqual(3, self.repo.byParents.call_count)

    def test_forFile_should_call_repo_with_list_not_set(self):
        from niprov.pipelinefactory import PipelineFactory
        p1a = self.fileWithLocation('p1a')
        p1b = self.fileWithLocation('p1b')
        t = self.fileWithLocation('t')
        t.provenance['parents'] = ['p1a','p1b']
        repodict = {'t':t,'p1a':p1a, 'p1b':p1b}
        self.repo.byLocations.side_effect = lambda ls: [repodict[l] for l in ls]
        self.repo.byParents.side_effect = lambda ls: []
        factory = PipelineFactory(dependencies=self.dependencies)
        with patch('niprov.pipelinefactory.Pipeline') as PipelineCtr:
            pipeline = factory.forFile(t)
            self.repo.byLocations.assert_called_with([])
            self.repo.byLocations.assert_any_call(['p1a','p1b'])

    def fileWithLocation(self, loc):
        f = Mock()
        f.location.toString.return_value = loc
        f.provenance = {}
        return f









from datetime import datetime
import numpy
from niprov.basefile import BaseFile
from niprov.libraries import Libraries


class ParrecFile(BaseFile):

    def __init__(self, location, **kwargs):
        super(ParrecFile, self).__init__(location, **kwargs)
        self.libs = self.dependencies.getLibraries()
        self.camera = self.dependencies.getCamera()

    def inspect(self):
        provenance = super(ParrecFile, self).inspect()
        img = self.libs.nibabel.load(self.path)
        info = img.header.general_info
        provenance['dimensions'] = list(img.shape)
        dateformat = '%Y.%m.%d / %H:%M:%S'
        acqstring = info['exam_date']
        provenance['acquired'] = datetime.strptime(acqstring, dateformat)
        provenance['subject'] = info['patient_name']
        provenance['subject-position'] = info['patient_position']
        provenance['protocol'] = info['protocol_name']
        provenance['technique'] = info['tech']
        tr = info['repetition_time']
        if isinstance(tr, numpy.ndarray):
            tr = tr.tolist()
        provenance['repetition-time'] = tr
        provenance['field-of-view'] = info['fov'].tolist()
        provenance['epi-factor'] = info['epi_factor']
        provenance['magnetization-transfer-contrast'] = bool(info['mtc'])
        provenance['diffusion'] = bool(info['diffusion'])
        provenance['duration'] = info['scan_duration']
        provenance['water-fat-shift'] = info['water_fat_shift']
        # per-image
        img0info = img.header.image_defs[0]
        provenance['slice-thickness'] = img0info['slice thickness']
        provenance['slice-orientation'] = img0info['slice orientation']
        provenance['echo-time'] = img0info['echo_time']
        provenance['flip-angle'] = img0info['image_flip_angle']
        provenance['inversion-time'] = img0info['Inversion delay']
        if provenance['diffusion']:
            provenance['modality'] = 'DWI'
        else:
            provenance['modality'] = 'MRI'
        self.camera.saveSnapshot(img.get_data(), for_=self)
        return provenance

    def getProtocolFields(self):
        return ['repetition-time', 'echo-time', 'flip-angle', 'epi-factor', 
                'water-fat-shift', 'subject-position']




from __future__ import print_function
import copy
from niprov.format import Format


class SimpleFormat(Format):

    _expectedFields = ['acquired','subject','protocol','dimensions','path']

    def serializeList(self, images):
        """Publish the provenance for several images on the commandline.

        Args:
            provenance (list): List of provenance dictionaries.
        """
        text = ''
        text += '\n'
        text += '{0:20} {1:12} {2:24} {3:20} {4:24}'.format(*self._expectedFields)
        for image in images:
            text += self.serializeSummary(image)
            text += '\n'
        text += '\n'
        return text

    def serializeSingle(self, img):
        """Publish the provenance for one image on the commandline.

        Args:
            provenance (dict): Provenance for one image file
        """
        text = ''
        text += '\n'
        for field, value in img.provenance.items():
            text += '{0:24} {1}\n'.format(field+':', str(value))
        text += '\n'
        return text

    def serializeSummary(self, image):
        """Publish a summary of the provenance for one image as one line in 
        the terminal.

        Args:
            provenance (dict): Provenance for one image file
        """
        provcopy = copy.deepcopy(image.provenance)
        for field in self._expectedFields:
            if field not in provcopy:
                provcopy[field] = None
        tmp = ('{0[acquired]!s}  {0[subject]!s:12} {0[protocol]!s:24} '
            '{0[dimensions]!s:20} {0[path]!s:24}')
        return tmp.format(provcopy)

    def serializeStatistics(self, stats):
        """Publish statistics for collected provenance in the terminal.

        Args:
            stats (dict): Dictionary with summary values.
        """
        text = ''
        text += '\n'
        text += ' Number of files: {0}\n'.format(stats['count'])
        text += ' Total file size: {0}\n'.format(stats['totalsize'])
        text += '\n'
        return text

    def serializePipeline(self, pipeline):
        """Pretty-print a pipeline to the terminal

        Args:
            pipeline (Pipeline): Pipeline object to publish.
        """
        tree = pipeline.asFilenameTree()
        def prettyPrintTree(tree, s='', lvl=0):
            for key, value in tree.items():
                s += '{0}+---{1}\n'.format('|   '*lvl,key)
                s = prettyPrintTree(value, s, lvl+1)
            return s
        return prettyPrintTree(tree)




import unittest
from mock import Mock, patch
from tests.ditest import DependencyInjectionTestBase


class ContextTests(DependencyInjectionTestBase):

    def setUp(self):
        super(ContextTests, self).setUp()
        patcher = patch('niprov.context.niprov')
        self.niprov = patcher.start()
        self.addCleanup(patcher.stop)
        from niprov import ProvenanceContext
        self.context = ProvenanceContext()
        self.context.deps = self.dependencies

    def test_get(self):
        self.assertEqual(self.context.get(), self.query)

    def test_add(self):
        self.context.add('file.p', True, {'c':3})
        self.niprov.adding.add.assert_called_with('file.p', True, {'c':3},
            self.dependencies)

    def test_log(self):
        self.context.log('new', 'trf', 'parents', 'code', 'logtext', False,
            'script', 'user', {'prov':1}, 'opts')
        self.niprov.logging.log.assert_called_with('new', 'trf', 'parents', 
            'code', 'logtext', False, 'script', 'user', {'prov':1}, 'opts',
            self.dependencies)

    def test_backup(self):
        self.context.backup()
        self.niprov.exporting.backup.assert_called_with(self.dependencies)

    def test_export(self):
        self.context.export('prov', 'medium', 'form')
        self.niprov.exporting.export.assert_called_with('prov', 'medium',
            'form', False, self.dependencies)

    def test_import(self):
        self.context.importp('fpath')
        self.niprov.importing.importp.assert_called_with('fpath',
            self.dependencies)

    def test_print(self):
        self.context.print_('images')
        self.niprov.exporting.print_.assert_called_with('images', False, 
            self.dependencies)

    def test_record(self):
        self.context.record('cmd', 'new', 'parents', True, 'args', 'kwargs', 
            'user', 'opts')
        self.niprov.recording.record.assert_called_with('cmd', 'new', 
            'parents', True, 'args', 'kwargs', 'user', 'opts',
            self.dependencies)

    def test_view(self):
        self.context.view('images', pipeline=True)
        self.niprov.exporting.view.assert_called_with('images', True, 
            self.dependencies)






import unittest, os
from mock import Mock, patch, sentinel
from tests.ditest import DependencyInjectionTestBase


class ComparerTests(DependencyInjectionTestBase):

    def setUp(self):
        super(ComparerTests, self).setUp()

    def test_compare_returns_Diff_object(self):
        from niprov.comparing import compare
        from niprov.diff import Diff
        out = compare(self.mockFile(), self.mockFile(), self.dependencies)
        self.assertIsInstance(out, Diff)

    def test_compare_creates_basic_diff(self):
        from niprov.comparing import compare
        with patch('niprov.comparing.Diff') as DiffCtor:
            compare(sentinel.f1, sentinel.f2, self.dependencies)
            DiffCtor.assert_called_with(sentinel.f1, sentinel.f2)

    def mockFile(self):
        f = Mock()
        f.getProvenance.return_value = {}
        return f




from tests.ditest import DependencyInjectionTestBase
from mock import Mock, sentinel, patch


class QueryTest(DependencyInjectionTestBase):

    def setUp(self):
        super(QueryTest, self).setUp()

    def test_byModality(self):
        from niprov.querying import Query
        q = Query(self.dependencies).byModality('magic')
        self.assertEqual(1, len(q.getFields()))
        self.assertEqual('modality', q.getFields()[0].name)
        self.assertEqual('magic', q.getFields()[0].value)

    def test_byProject(self):
        from niprov.querying import Query
        q = Query(self.dependencies).byProject('sealion')
        self.assertEqual(1, len(q.getFields()))
        self.assertEqual('project', q.getFields()[0].name)
        self.assertEqual('sealion', q.getFields()[0].value)

    def test_byUser(self):
        from niprov.querying import Query
        q = Query(self.dependencies).byUser('dumbledore')
        self.assertEqual(1, len(q.getFields()))
        self.assertEqual('user', q.getFields()[0].name)
        self.assertEqual('dumbledore', q.getFields()[0].value)

    def test_bySubject(self):
        from niprov.querying import Query
        q = Query(self.dependencies).bySubject('potter, h')
        self.assertEqual(1, len(q.getFields()))
        self.assertEqual('subject', q.getFields()[0].name)
        self.assertEqual('potter, h', q.getFields()[0].value)

    def test_byApproval(self):
        from niprov.querying import Query
        q = Query(self.dependencies).byApproval('ceterum censeo')
        self.assertEqual(1, len(q.getFields()))
        self.assertEqual('approval', q.getFields()[0].name)
        self.assertEqual('ceterum censeo', q.getFields()[0].value)

    def test_Iter_returns_repository_inquire_results(self):
        from niprov.querying import Query
        self.repo.inquire.return_value = [sentinel.r1, sentinel.r2]
        q = Query(self.dependencies)
        out = list(q)
        self.repo.inquire.assert_called_with(q)
        self.assertEqual(out, self.repo.inquire())

    def test_Len_on_query(self):
        from niprov.querying import Query
        self.repo.inquire.return_value = [sentinel.r1, sentinel.r2]
        q = Query(self.dependencies)
        self.assertEqual(len(q), 2)

    def test_In_on_query(self):
        from niprov.querying import Query
        self.repo.inquire.return_value = [sentinel.r1, sentinel.r2]
        q = Query(self.dependencies).bySubject('abc')
        self.assertTrue(sentinel.r2 in q)

    def test_Len_or_Iter_or_in_only_runs_inquire_once(self):
        from niprov.querying import Query
        self.repo.inquire.return_value = [sentinel.r1, sentinel.r2]
        q = Query(self.dependencies)
        len(q)
        list(q)
        sentinel.r2 in q
        self.assertEqual(self.repo.inquire.call_count, 1)

    def test_byLocation(self):
        self.locationFactory.completeString.side_effect = lambda s: s
        from niprov.querying import Query
        result = Query(self.dependencies).byLocation('abc')
        self.repo.byLocation.assert_called_with('abc')
        self.assertEqual(result, self.repo.byLocation('abc'))

    def test_Statistics(self):
        from niprov.querying import Query
        result = Query(self.dependencies).statistics()
        self.repo.statistics.assert_called_with()
        self.assertEqual(result, self.repo.statistics())

    def test_Latest(self):
        from niprov.querying import Query
        result = Query(self.dependencies).latest()
        self.repo.latest.assert_called_with()
        self.assertEqual(result, self.repo.latest())

    def test_All(self):
        from niprov.querying import Query
        result = Query(self.dependencies).all()
        self.repo.all.assert_called_with()
        self.assertEqual(result, self.repo.all())

    def test_Completes_locationString_byLocation(self):
        from niprov.querying import Query
        result = Query(self.dependencies).byLocation('abc')
        self.locationFactory.completeString.assert_any_call('abc')
        self.repo.byLocation.assert_called_with(self.locationFactory.completeString())

    def test_allModalities(self):
        from niprov.querying import Query
        q = Query(self.dependencies).allModalities()
        self.assertEqual('modality', q.getFields()[0].name)
        self.assertTrue(q.getFields()[0].all)

    def test_allUsers(self):
        from niprov.querying import Query
        q = Query(self.dependencies).allUsers()
        self.assertEqual('user', q.getFields()[0].name)
        self.assertTrue(q.getFields()[0].all)

    def test_allProjects(self):
        from niprov.querying import Query
        q = Query(self.dependencies).allProjects()
        self.assertEqual('project', q.getFields()[0].name)
        self.assertTrue(q.getFields()[0].all)

    def test_copiesOf_returns_empty_list_for_size_0_file(self):
        from niprov.querying import Query
        target = Mock()
        target.provenance = {'size':0}
        results = Query(self.dependencies).copiesOf(target)
        self.assertEqual(0, len(results))

    def test_copiesOf(self):
        from niprov.querying import Query
        target = Mock()
        target.provenance = {'hash':'a7b8c9', 'size':1}
        q = Query(self.dependencies).copiesOf(target)
        self.assertEqual(1, len(q.getFields()))
        self.assertEqual('hash', q.getFields()[0].name)
        self.assertEqual('a7b8c9', q.getFields()[0].value)






from os.path import basename

class Diff(object):
    """Difference between two files.

    This represents differences in provenance between two files.

    See :py:mod:`niprov.comparing`

    Args:
        file1 (:class:`.BaseFile`): One of two niprov BaseFile objects to 
            compare.
        file2 (:class:`.BaseFile`): As file1
    """

    NCHARSCOL = 20              # width of columns
    defaultIgnore = ['_id']     # these fields are always ignored

    def __init__(self, file1, file2):
        self.file1 = file1
        self.file2 = file2

    def getDifferences(self, ignore=None, select=None):
        """Get dictionary with fields that differ and how they differ.

        Args:
            ignore (list): Optional. List of fields not to evaluate when 
                determining differences.
            select (list): Optional. List of fields that should be specifically
                evaluated. All other fields will be ignored.

        Returns:
            dict: A dictionary with provenance fields as keys and strings
                indicating how they differ.
        """
        assert isinstance(ignore, list) or ignore is None
        if ignore is None:
            ignore = []
        ignore += self.defaultIgnore
        prov1 = self.file1.getProvenance()
        prov2 = self.file2.getProvenance()
        if select:
            allkeys = set(prov1.keys()+prov2.keys())
            ignore = [k for k in allkeys if k not in select]
        diffDict = {}
        for k in set(prov1.keys()).difference(prov2.keys()):
            if k not in ignore:
                diffDict[k] = 'missingIn2'
        for k in set(prov2.keys()).difference(prov1.keys()):
            if k not in ignore:
                diffDict[k] = 'missingIn1'
        for k in set(prov1.keys()).intersection(prov2.keys()):
            if k not in ignore:
                if prov1[k] != prov2[k]:
                    diffDict[k] = 'value'
        return diffDict

    def getSame(self):
        """Get dictionary with fields that have equal values.

        Returns:
            dict: A dictionary with provenance fields as keys the string
                  'same' as value.
        """
        prov1 = self.file1.getProvenance()
        prov2 = self.file2.getProvenance()
        sameDict = {}
        for k in set(prov1.keys()).intersection(prov2.keys()):
            if prov1[k] == prov2[k]:
                sameDict[k] = 'same'
        return sameDict

    def getDifferenceString(self, ignore=None, select=None):
        """Get table of differences as string.

        Args:
            ignore (list): Optional. List of fields not to evaluate when 
                determining differences.
            select (list): Optional. List of fields that should be specifically
                evaluated. All other fields will be ignored.

        Returns:
            str: A three-columns table listing provenance fields and their
                respective values for the two files.
        """
        differences = self.getDifferences(ignore, select)
        return self._tableStringFromDiffDict(differences)

    def getSameString(self):
        """Get table of values that are the same for the compared files.

        Returns:
            str: A three-columns table listing provenance fields and their
                respective values for the two files.
        """
        same = self.getSame()
        return self._tableStringFromDiffDict(same)

    def _tableStringFromDiffDict(self, diffDict):
        if not diffDict:
            return ''
        name1 = basename(str(self.file1.location))
        name2 = basename(str(self.file2.location))
        prov1 = self.file1.getProvenance()
        prov2 = self.file2.getProvenance()
        def row(*vals):
            cells = [c[:self.NCHARSCOL] for c in vals]
            cells = [c.ljust(self.NCHARSCOL) for c in cells]
            return ' '.join(cells)+'\n'
        diffStr = 'Differences:\n'
        diffStr += row('', name1, name2)
        for field, status in diffDict.items():
            val1 = prov1.get(field, 'n/a')
            val2 = prov2.get(field, 'n/a')
            diffStr += row(field, str(val1), str(val2))
        return diffStr

    def areEqual(self, ignore=None, select=None):
        """Whether there are any differences between the files.

        Args:
            ignore (list): Optional. List of fields not to evaluate when 
                determining differences.
            select (list): Optional. List of fields that should be specifically
                evaluated. All other fields will be ignored.

        Returns:
            bool: True if no differences, False otherwise.
        """
        differences = self.getDifferences(ignore, select)
        return len(differences) == 0

    def areEqualProtocol(self):
        """Whether there are any differences for protocol fields.

        Each :class:`.BaseFile` subtype has a getProtocolFields() method
        that is used here to selectively see if any of these are different.

        Returns:
            bool: True if no differences, False otherwise.
        """
        protocol = self.file1.getProtocolFields()
        differences = self.getDifferences(select=protocol)
        return len(differences) == 0

    def assertEqual(self, ignore=None, select=None):
        """Raises exception if there are differences.

        Args:
            ignore (list): Optional. List of fields not to evaluate when 
                determining differences.
            select (list): Optional. List of fields that should be specifically
                evaluated. All other fields will be ignored.

        Raises:
            AssertionError: Message with differences in a table.
        """
        differences = self.getDifferenceString(ignore, select)
        if differences:
            raise AssertionError(differences)

    def assertEqualProtocol(self):
        """Raises exception if there are differences in protocol fields.

        Each :class:`.BaseFile` subtype has a getProtocolFields() method
        that is used here to selectively see if any of these are different.

        Raises:
            AssertionError: Message with protocol differences in a table.
        """
        protocol = self.file1.getProtocolFields()
        differences = self.getDifferenceString(select=protocol)
        if differences:
            raise AssertionError(differences)

    def __str__(self):
        return self.getDifferenceString()



from niprov.dependencies import Dependencies
from niprov.basefile import BaseFile
from niprov.pipeline import Pipeline


class Format(object):
    """Parent Format class from which specific formats are derived.
    """

    def __init__(self, dependencies=Dependencies()):
        self.fileExtension = 'txt'

    def serialize(self, provenance):
        """Publish provenance.

        This determines if the provenance is for a single file or multiple,
        and then calls the appropriate more specific serialize method.
        """
        if isinstance(provenance, Pipeline):
            return self.serializePipeline(provenance)
        elif isinstance(provenance, dict):
            return self.serializeStatistics(provenance)
        elif hasattr(provenance, '__iter__'):
            return self.serializeList(provenance)
        else:
            return self.serializeSingle(provenance)

    def serializeList(self, provenance):
        raise NotImplementedError('serializeList')

    def serializeSingle(self, provenance):
        raise NotImplementedError('serializeSingle')

    def serializeStatistics(self, provenance):
        raise NotImplementedError('serializeStatistics')

    def serializePipeline(self, pipeline):
        raise NotImplementedError('serializePipeline')



from niprov.formatxml import XmlFormat
from niprov.formatjson import JsonFormat
from niprov.pictures import PictureCache


class FileMedium(object):

    def __init__(self, dependencies):
        self.filesys = dependencies.getFilesystem()
        self.clock = dependencies.getClock()
        self.listener = dependencies.getListener()

    def export(self, formattedProvenance, form):
        if isinstance(form, PictureCache):
            fname = formattedProvenance
        else:
            fname = 'provenance_{0}.{1}'.format(self.clock.getNowString(),
                                                form.fileExtension)
            self.filesys.write(fname, formattedProvenance)
        self.listener.exportedToFile(fname)
        return fname



import unittest, os
from mock import Mock, patch, call
from tests.ditest import DependencyInjectionTestBase


class CommandlineTests(DependencyInjectionTestBase):

    def setUp(self):
        super(CommandlineTests, self).setUp()

    def test_Only_prints_if_appropriate_verbosity_setting(self):
        from niprov.commandline import Commandline
        self.dependencies.config.verbosity = 'warning'
        with patch('__builtin__.print') as mprint:
            cmd = Commandline(self.dependencies)
            cmd.log('info','Do you remember..')
            assert not mprint.called
            cmd.log('warning','Watch out!')
            mprint.assert_called_with('[provenance:warning] Watch out!')

    def test_exportedToFile_logs_info(self):
        from niprov.commandline import Commandline
        self.dependencies.config.verbosity = 'info'
        cmd = Commandline(self.dependencies)
        cmd.log = Mock()
        cmd.exportedToFile('backupfile.x')
        cmd.log.assert_called_with('info','Exported to file: backupfile.x')

    def test_addUnknownParent(self):
        from niprov.commandline import Commandline
        self.dependencies.config.verbosity = 'info'
        cmd = Commandline(self.dependencies)
        cmd.log = Mock()
        cmd.addUnknownParent('backupfile.x')
        cmd.log.assert_called_with('warning', 'backupfile.x unknown. Adding to provenance')

    def test_fileAdded(self):
        from niprov.commandline import Commandline
        self.dependencies.config.verbosity = 'info'
        cmd = Commandline(self.dependencies)
        cmd.log = Mock()
        img = Mock()
        img.path = 'xyz'
        img.status = 'new'
        cmd.fileAdded(img)
        cmd.log.assert_called_with('info', 'New file: xyz')
        img.status = 'series-new-file'
        img.provenance = {'filesInSeries':[1, 2, 3]}
        img.getSeriesId.return_value = 'series987'
        cmd.fileAdded(img)
        cmd.log.assert_called_with('info', 'Added 3rd file to series: series987')
        img.status = 'new-version'
        cmd.fileAdded(img)
        cmd.log.assert_called_with('info', 'Added new version for: xyz')

    def test_usingCopyAsParent(self):
        from niprov.commandline import Commandline
        self.dependencies.config.verbosity = 'info'
        cmd = Commandline(self.dependencies)
        cmd.log = Mock()
        copy = Mock()
        copy.location = 'moon:/copy/location'
        cmd.usingCopyAsParent(copy)
        cmd.log.assert_called_with('warning', 
            'Used provenance from copy found at '+str(copy.location))
        




import unittest
from mock import Mock, patch, sentinel
from tests.ditest import DependencyInjectionTestBase


class MongoRepoTests(DependencyInjectionTestBase):

    def setUp(self):
        super(MongoRepoTests, self).setUp()
        self.pictureCache.getBytes.return_value = None
        self.db = Mock()
        self.db.provenance.find_one.return_value = {}
        self.db.provenance.find.return_value = {}
        self.pymongo = None

    def setupRepo(self):
        from niprov.mongo import MongoRepository
        with patch('niprov.mongo.pymongo') as self.pymongo:
            self.repo = MongoRepository(dependencies=self.dependencies)
        self.repo.db = self.db

    def test_Connection(self):
        from niprov.mongo import MongoRepository
        with patch('niprov.mongo.pymongo') as pymongo:
            repo = MongoRepository(dependencies=self.dependencies)
        pymongo.MongoClient.assert_called_with(self.config.database_url)
        self.assertEqual(pymongo.MongoClient().get_default_database(), repo.db)

    def test_byLocation_returns_img_from_record_with_path(self):
        self.setupRepo()
        p = '/p/f1'
        out = self.repo.byLocation(p)
        self.db.provenance.find_one.assert_called_with({'location':p})
        self.fileFactory.fromProvenance.assert_called_with(
            self.db.provenance.find_one())
        self.assertEqual(self.fileFactory.fromProvenance(), out)

    def test_getSeries(self):
        self.setupRepo()
        img = Mock()
        out = self.repo.getSeries(img)
        self.db.provenance.find_one.assert_called_with(
            {'seriesuid':img.getSeriesId()})
        self.fileFactory.fromProvenance.assert_called_with(
            self.db.provenance.find_one())
        self.assertEqual(self.fileFactory.fromProvenance(), out)

    def test_getSeries_returns_None_right_away_if_no_series_id(self):
        self.setupRepo()
        img = Mock()
        img.getSeriesId.return_value = None
        out = self.repo.getSeries(img)
        assert not self.db.provenance.find_one.called
        self.assertEqual(None, out)

    def test_Add(self):
        self.setupRepo()
        img = Mock()
        img.provenance = {'a':1, 'b':2}
        self.repo.add(img)
        self.db.provenance.insert_one.assert_called_with({'a':1, 'b':2})

    def test_update(self):
        self.setupRepo()
        img = Mock()
        img.provenance = {'a':1, 'b':2}
        self.repo.update(img)
        self.db.provenance.update.assert_called_with(
            {'location':img.location.toString()}, {'a':1, 'b':2})

    def test_all(self):
        self.fileFactory.fromProvenance.side_effect = lambda p: 'img_'+p
        self.db.provenance.find.return_value = ['p1', 'p2']
        self.setupRepo()
        out = self.repo.all()
        self.db.provenance.find.assert_called_with()
        self.fileFactory.fromProvenance.assert_any_call('p1')
        self.fileFactory.fromProvenance.assert_any_call('p2')
        self.assertEqual(['img_p1', 'img_p2'], out)

    def test_updateApproval(self):
        self.setupRepo()
        img = Mock()
        p = '/p/f1'
        newStatus = 'oh-oh'
        self.repo.updateApproval(p, newStatus)
        self.db.provenance.update.assert_called_with(
            {'location':p}, {'$set': {'approval': newStatus}})

    def test_latest(self):
        self.fileFactory.fromProvenance.side_effect = lambda p: 'img_'+p
        self.db.provenance.find.return_value = Mock()
        self.db.provenance.find.return_value.sort.return_value.limit.return_value = ['px','py']
        self.setupRepo()
        out = self.repo.latest()
        self.db.provenance.find.assert_called_with()
        self.db.provenance.find().sort.assert_called_with('added', -1)
        self.db.provenance.find().sort().limit.assert_called_with(20)
        self.fileFactory.fromProvenance.assert_any_call('px')
        self.fileFactory.fromProvenance.assert_any_call('py')
        self.assertEqual(['img_px', 'img_py'], out)

    def test_statistics(self):
        self.db.provenance.aggregate.return_value = [sentinel.stats,]
        self.setupRepo()
        out = self.repo.statistics()
        self.db.provenance.aggregate.assert_called_with(
           [{'$group':
                 {
                   '_id': None,
                   'totalsize': { '$sum': '$size' },
                   'count': { '$sum': 1 }
                 }
            }])
        self.assertEqual(sentinel.stats, out)

    def test_statistics_if_no_records(self):
        self.db.provenance.aggregate.return_value = []
        self.setupRepo()
        out = self.repo.statistics()
        self.assertEqual({'count':0}, out)

    def test_byId(self):
        self.setupRepo()
        ID = 'abc123'
        out = self.repo.byId(ID)
        self.db.provenance.find_one.assert_called_with({'id':ID})
        self.fileFactory.fromProvenance.assert_called_with(
            self.db.provenance.find_one())
        self.assertEqual(self.fileFactory.fromProvenance(), out)

    def test_If_db_returns_None_should_return_None_for_byId_byLocation(self):
        self.setupRepo()
        self.db.provenance.find_one.return_value = None
        out = self.repo.byId('abc123')
        assert not self.fileFactory.fromProvenance.called
        self.assertEqual(None, out)

    def test_byLocations(self):
        self.fileFactory.fromProvenance.side_effect = lambda p: 'img_'+p
        self.db.provenance.find.return_value = ['p1', 'p2']
        self.setupRepo()
        out = self.repo.byLocations(['l1','l2'])
        self.db.provenance.find.assert_called_with({'location':{'$in':['l1','l2']}})
        self.fileFactory.fromProvenance.assert_any_call('p1')
        self.fileFactory.fromProvenance.assert_any_call('p2')
        self.assertEqual(['img_p1', 'img_p2'], out)

    def test_byParents(self):
        self.fileFactory.fromProvenance.side_effect = lambda p: 'img_'+p
        self.db.provenance.find.return_value = ['p1', 'p2']
        self.setupRepo()
        out = self.repo.byParents(['x1','x2'])
        self.db.provenance.find.assert_called_with({'parents':{'$in':['x1','x2']}})
        self.fileFactory.fromProvenance.assert_any_call('p1')
        self.fileFactory.fromProvenance.assert_any_call('p2')
        self.assertEqual(['img_p1', 'img_p2'], out)

    def test_Obtains_optional_snapshot_data_from_cache_when_serializing(self):
        self.pictureCache.getBytes.return_value = sentinel.snapbytes
        with patch('niprov.mongo.bson') as bson:
            bson.Binary.return_value = sentinel.snapbson
            self.setupRepo()
            img = Mock()
            img.provenance = {'a':1}
            self.repo.add(img)
            self.pictureCache.getBytes.assert_called_with(for_=img)
            bson.Binary.assert_called_with(sentinel.snapbytes)
            self.db.provenance.insert_one.assert_called_with({'a':1, 
                '_snapshot-data':sentinel.snapbson})

    def test_If_no_snapshot_doesnt_add_data_field(self):
        self.pictureCache.getBytes.return_value = None
        with patch('niprov.mongo.bson') as bson:
            self.setupRepo()
            img = Mock()
            img.provenance = {'a':1}
            self.repo.add(img)
            assert not bson.Binary.called
            self.db.provenance.insert_one.assert_called_with({'a':1})

    def test_If_snapshotdata_hands_them_to_pictureCache_on_deserializing(self):
        img = Mock()
        self.fileFactory.fromProvenance.return_value = img
        self.setupRepo()
        self.db.provenance.find_one.return_value = {'a':3}
        out = self.repo.byLocation('/p/f1')
        assert not self.pictureCache.keep.called
        self.db.provenance.find_one.return_value = {'a':3, 
                                                    '_snapshot-data':'y7yUyS'}
        out = self.repo.byLocation('/p/f1')
        self.pictureCache.keep.assert_called_with('y7yUyS', for_=img)

    def test_Query(self):
        self.db.provenance.find.return_value = ['record1']
        self.setupRepo()
        q = Mock()
        field1 = Mock()
        field1.name = 'color'
        field1.value = 'red'
        field1.all = False
        q.getFields.return_value = [field1]
        out = self.repo.inquire(q)
        self.db.provenance.find.assert_called_with({'color':'red'})
        self.fileFactory.fromProvenance.assert_called_with('record1')

    def test_Ensures_text_index_for_search(self):
        self.setupRepo()
        self.repo.search('')
        searchfields = ['location','user','subject','project','protocol',
                  'transformation','technique','modality']
        indexspec = [(field, 'text') for field in searchfields]
        self.db.provenance.create_index.assert_called_with(indexspec, 
                                                           name='textsearch')

    def test_Search(self):
        self.db.provenance.find.return_value = ['r1','r2']
        self.setupRepo()
        self.repo.search('xyz')
        self.db.provenance.find.assert_called_with({'$text':{'$search': 'xyz'}})
        self.fileFactory.fromProvenance.assert_any_call('r1')
        self.fileFactory.fromProvenance.assert_any_call('r2')

    def test_Query_for_ALL_field(self):
        self.db.provenance.distinct.return_value = ['r1','r2']
        self.setupRepo()
        q = Mock()
        field1 = Mock()
        field1.name = 'color'
        field1.all = True
        q.getFields.return_value = [field1]
        out = self.repo.inquire(q)
        self.db.provenance.distinct.assert_called_with('color')
        assert not self.fileFactory.fromProvenance.called




from niprov.format import Format
import io, os

_CACHE = {}

class PictureCache(Format):

    def __init__(self, dependencies):
        cachedir = os.path.expanduser('~/.niprov-snapshots')
        if not os.path.isdir(cachedir):
            os.mkdir(cachedir)

    def new(self):
        return io.BytesIO()

    def keep(self, picture, for_):
        imgId = for_.provenance['id']
        if hasattr(picture, 'read'):
            picture.seek(0)
            bytes = picture.read()
        else:
            bytes = str(picture)
        _CACHE[imgId] = bytes

    def getBytes(self, for_):
        imgId = for_.provenance['id']
        if imgId in _CACHE:
            return _CACHE[imgId]
        return None

    def getFilepath(self, for_):
        return self.saveToDisk(for_)

    def saveToDisk(self, for_):
        imgId = for_.provenance['id']
        fpath = os.path.expanduser('~/.niprov-snapshots/{}.png'.format(imgId))
        if os.path.isfile(fpath):
            return fpath
        elif imgId in _CACHE:
            with open(fpath, 'w') as picfile:
                picfile.write(_CACHE[imgId])
            return fpath
        else:
            return None

    def serializeSingle(self, image):
        """Provides file path to picture of image.

        This is part of the :class:`.Format` interface. 
        """
        return self.getFilepath(for_=image)




from niprov.dependencies import Dependencies


class Camera(object):

    def __init__(self, dependencies):
        self.film = dependencies.getPictureCache()
        self.libs = dependencies.getLibraries()

    def saveSnapshot(self, data, for_):
        """Plot an overview of the image and store it.

        Uses :class:`.PictureCache` as service that provides a file-like 
                handle to save the plotted picture to.
        Calls takeSnapshot() to do the actual plotting.

        Args:
            data (numpy.ndarray): Array of 2, 3 or 4 dimensions with image data.
        """
        newPicture = self.film.new()
        success = self.takeSnapshot(data, on=newPicture)
        if success:
            self.film.keep(newPicture, for_)

    def takeSnapshot(self, data, on):
        """Plot an overview of the image using matplotlib.pyplot.

        Args:
            data (numpy.ndarray): Array of 2, 3 or 4 dimensions with image data.
            on (str or file-like object): Where to save figure to.
        """
        if not self.libs.hasDependency('pyplot'):
            return False
        plt = self.libs.pyplot

        try:
            ndims = len(data.shape)
            sliceOrder = [1, 0, 2]
            fig, axs = plt.subplots(nrows=1, ncols=ndims, figsize=(8, 3), dpi=100)
            for d in range(ndims):
                slicing = [slice(None)]*ndims
                slicing[sliceOrder[d]] = int(data.shape[d]/2)
                axs[d].matshow(data[slicing].T, origin='lower', 
                    cmap = plt.get_cmap('gray'), vmin = 0, vmax = data.max())
                axs[d].locator_params(nbins=3)
                axs[d].tick_params(axis='both', which='major', labelsize=8)
            plt.tight_layout()
            plt.savefig(on)
        except Exception as e:
            return False
        return True



from unittest import TestCase
from mock import Mock, sentinel, patch
import io, os, shutil, bson


class PictureCacheTests(TestCase):

    def setUp(self):
        picdir = os.path.expanduser('~/.niprov-snapshots')
        if os.path.isdir(picdir):
            shutil.rmtree(picdir)
        import niprov.pictures
        niprov.pictures._CACHE = {} # reset cache

    def test_Serialize(self):
        from niprov.pictures import PictureCache
        pictures = PictureCache(sentinel.dependencies)
        pictures.getFilepath = Mock()
        fpath = pictures.serializeSingle(sentinel.img)
        pictures.getFilepath.assert_called_with(for_=sentinel.img)
        self.assertEqual(pictures.getFilepath(), fpath)

    def test_Provides_new_picture_file_handle(self):
        from niprov.pictures import PictureCache
        pictures = PictureCache(sentinel.dependencies)
        newPicture = pictures.new()
        self.assertTrue(hasattr(newPicture, 'write'))

    def test_Stored_picture_can_be_retrieved_as_filepath(self):
        from niprov.pictures import PictureCache
        myImg = Mock()
        myImg.provenance = {'id':'007'}
        pictures = PictureCache(sentinel.dependencies)
        self.assertIsNone(pictures.getFilepath(for_=myImg))
        pictures.keep(io.BytesIO('/x10/x05/x5f'), for_=myImg)
        picfpath = os.path.expanduser('~/.niprov-snapshots/007.png')
        self.assertEqual(picfpath, pictures.getFilepath(for_=myImg))

    def test_Stored_picture_can_be_retrieved_as_bytes(self):
        from niprov.pictures import PictureCache
        myImg = Mock()
        myImg.provenance = {'id':'007'}
        pictures = PictureCache(sentinel.dependencies)
        newPicture = pictures.new()
        newPicture.write('/x10/x05/x5f')
        pictures.keep(newPicture, for_=myImg)
        outBytes = pictures.getBytes(for_=myImg)
        self.assertEqual('/x10/x05/x5f', outBytes)

    def test_Can_be_told_to_persist_picture_to_disk_now(self):
        from niprov.pictures import PictureCache
        myImg = Mock()
        myImg.provenance = {'id':'007'}
        pictures = PictureCache(sentinel.dependencies)
        pictures.saveToDisk(for_=myImg) #shouldn't do anything
        pictures.keep(io.BytesIO('/x10/x05/x5f'), for_=myImg)
        pictures.saveToDisk(for_=myImg)
        picfpath = os.path.expanduser('~/.niprov-snapshots/007.png')
        self.assertTrue(os.path.isfile(picfpath))
        with open(picfpath) as picFile:
            self.assertEqual(picFile.read(), '/x10/x05/x5f')

    def test_If_pic_is_not_in_cache_but_on_filesystem_provides_that(self):
        import niprov.pictures
        myImg = Mock()
        myImg.provenance = {'id':'007'}
        pictures = niprov.pictures.PictureCache(sentinel.dependencies)
        pictures.keep(io.BytesIO('/x10/x05/x5f'), for_=myImg)
        pictures.saveToDisk(for_=myImg)
        niprov.pictures._CACHE = {} # reset cache
        picfpath = os.path.expanduser('~/.niprov-snapshots/007.png')
        self.assertEqual(picfpath, pictures.getFilepath(for_=myImg))

    def test_Keep_accepts_bsonBinary(self):
        from niprov.pictures import PictureCache
        myImg = Mock()
        myImg.provenance = {'id':'007'}
        pictures = PictureCache(sentinel.dependencies)
        pictures.keep(bson.Binary('/x10/x05/x5f'), for_=myImg)
        self.assertEqual('/x10/x05/x5f', pictures.getBytes(for_=myImg))
        
    




from niprov.dependencies import Dependencies
from niprov.config import Configuration
import niprov


class ProvenanceContext(object):

    def __init__(self):
        self.config = Configuration()
        self.deps = Dependencies(self.config)

    def add(self, filepath, transient=False, provenance=None):
        """See :py:mod:`niprov.adding`  """
        return niprov.adding.add(filepath, transient, provenance, self.deps)

    def approve(self, filepath):
        """See :py:mod:`niprov.approval`  """
        return niprov.approval.approve(filepath, dependencies=self.deps)

    def backup(self):
        """See :py:mod:`niprov.exporting`  """
        return niprov.exporting.backup(self.deps)

    def compare(self, file1, file2):
        """See :py:mod:`niprov.comparing`  """
        return niprov.comparing.compare(file1, file2, dependencies=self.deps)

    def discover(self, root):
        """See :py:mod:`niprov.discovery`  """
        return niprov.discovery.discover(root, dependencies=self.deps)

    def export(self, images, medium, form, pipeline=False):
        """See :py:mod:`niprov.exporting`  """
        return niprov.exporting.export(images, medium, form, pipeline, 
            self.deps)

    def get(self):
        """See :py:mod:`niprov.querying`  """
        return self.deps.getQuery()

    def importp(self, fpath):
        """See :py:mod:`niprov.importing`  """
        return niprov.importing.importp(fpath, self.deps)

    def inspect(self, location):
        """See :py:mod:`niprov.inspection`  """
        return niprov.inspection.inspect(location, dependencies=self.deps)

    def log(self, new, transformation, parents, code=None, logtext=None, 
            transient=False, script=None, user=None, provenance=None, 
            opts=None):
        """See :py:mod:`niprov.logging`  """
        return niprov.logging.log(new, transformation, parents, code, logtext,
            transient, script, user, provenance, opts, self.deps)

    def markForApproval(self, files):
        """See :py:mod:`niprov.approval`  """
        return niprov.approval.markForApproval(files, dependencies=self.deps)

    def markedForApproval(self):
        """See :py:mod:`niprov.approval`  """
        return niprov.approval.markedForApproval(dependencies=self.deps)

    def print_(self, images, pipeline=False):
        """See :py:mod:`niprov.exporting`  """
        return niprov.exporting.print_(images, pipeline, self.deps)

    def renameDicoms(self, dicomdir):
        """See :py:mod:`niprov.renaming`  """
        return niprov.renaming.renameDicoms(dicomdir, dependencies=self.deps)

    def record(self, command, new=None, parents=None, transient=False, 
        args=None, kwargs=None, user=None, opts=None):
        """See :py:mod:`niprov.recording`  """
        return niprov.recording.record(command, new, parents, transient, 
            args, kwargs, user, opts, self.deps)

    def search(self, text):
        """See :py:mod:`niprov.searching`  """
        return niprov.searching.search(text, dependencies=self.deps)

    def selectApproved(self, files):
        """See :py:mod:`niprov.approval`  """
        return niprov.approval.selectApproved(files, dependencies=self.deps)

    def view(self, images, pipeline=False):
        """See :py:mod:`niprov.exporting`  """
        return niprov.exporting.view(images, pipeline, self.deps)




#!/usr/bin/python
# -*- coding: UTF-8 -*-
from niprov.dependencies import Dependencies


def export(provenance, medium, form, pipeline=False, dependencies=Dependencies()):
    """Publish or simply return provenance for selected files.

    To get provenance on one specific file, pass its path as the 'forFile' 
    argument. Alternatively, to get all files associated with a certain subject,
    use the 'forSubject' argument. If none of these is used, provenance for the 
    most recently registered files is reported.

    Args:
        provenance: Niprov BaseFile object or list of such.
        medium (str): The medium in which to publish the provenance. 
            One of:
                'stdout'  (print the provenance to the terminal), 
                'direct'  (return object to caller), 
                'file'    (write to a text file),
                'viewer'  (open in the system image viewer).
        form (str): The format in which to serialize the provenance. 
            One of 'json','xml','narrated','simple','dict','picture'.

    Returns:
        Depends on medium selected. 
    """
    formatFactory = dependencies.getFormatFactory()
    mediumFactory = dependencies.getMediumFactory()
    makePipeline = dependencies.getPipelineFactory()

    form = formatFactory.create(form)
    medium = mediumFactory.create(medium)
    
    if pipeline:
        provenance = makePipeline.forFile(provenance)

    formattedProvenance = form.serialize(provenance)
    return medium.export(formattedProvenance, form)

def print_(provenance, pipeline=False, dependencies=Dependencies()):
    """Shortcut for export(medium='stdout', form='simple').
    """
    return export(provenance, medium='stdout', form='simple',
                  pipeline=pipeline, dependencies=dependencies)

def backup(dependencies=Dependencies()):
    """Shortcut for export(medium='file', form='json') for all provenance.
    """
    provenance = dependencies.getQuery().all()
    return export(provenance, medium='file', form='json', 
                  dependencies=dependencies)

def view(provenance, pipeline=False, dependencies=Dependencies()):
    """Shortcut for export(medium='viewer', form='picture').
    """
    return export(provenance, medium='viewer', form='picture', 
        pipeline=pipeline, dependencies=dependencies)




import unittest
from mock import Mock, patch, call
import os
from tests.ditest import DependencyInjectionTestBase


class UsersTests(DependencyInjectionTestBase):

    def setUp(self):
        super(UsersTests, self).setUp()
        self.config.user = None

    def test_determineUser_if_not_config_or_runtime_uses_OS(self):
        from niprov.users import Users
        users = Users(self.dependencies)
        import getpass
        self.assertEqual(users.determineUser(None), getpass.getuser())

    def test_determineUser_if_not_runtime_uses_config(self):
        from niprov.users import Users
        users = Users(self.dependencies)
        self.config.user = 'jasmine'
        self.assertEqual(users.determineUser(None), 'jasmine')

    def test_determineUser_if_runtime_given_uses_that(self):
        from niprov.users import Users
        users = Users(self.dependencies)
        self.config.user = 'jasmine'
        self.assertEqual(users.determineUser('oddyseus'), 'oddyseus')




#!/usr/bin/python
# -*- coding: UTF-8 -*-
import webbrowser


class ViewerMedium(object):
    """Uses the system picture viewer to display an picture file.
    """

    def __init__(self, dependencies):
        pass

    def export(self, formattedProvenance, form=None):
        if formattedProvenance is None:
            return
        webbrowser.open(formattedProvenance)




#!/usr/bin/python
# -*- coding: UTF-8 -*-
from mock import Mock, sentinel, patch
from tests.ditest import DependencyInjectionTestBase


class FileMediumTests(DependencyInjectionTestBase):

    def test_Writes_formattedProvenance_to_file(self):
        from niprov.mediumfile import FileMedium
        medium = FileMedium(self.dependencies)
        out = medium.export('Once upon a time..', self.format)
        self.assertEqual('Once upon a time..', 
            self.filesys.write.call_args[0][1])

    def test_Comes_up_with_filename(self):
        from niprov.mediumfile import FileMedium
        self.clock.getNowString.return_value = 'hammertime'
        medium = FileMedium(self.dependencies)
        out = medium.export('provstr', self.format)
        self.filesys.write.assert_called_with('provenance_hammertime.txt',
            'provstr')

    def test_Returns_filename(self):
        from niprov.mediumfile import FileMedium
        self.clock.getNowString.return_value = 'hammertime'
        medium = FileMedium(self.dependencies)
        out = medium.export('provstr', self.format)
        self.assertEqual('provenance_hammertime.txt', out)

    def test_Reports_filename_to_listener(self):
        from niprov.mediumfile import FileMedium
        self.clock.getNowString.return_value = 'hammertime'
        medium = FileMedium(self.dependencies)
        out = medium.export('provstr', self.format)
        self.listener.exportedToFile.assert_called_with(
            'provenance_hammertime.txt')

    def test_Specific_extension_for_format(self):
        from niprov.mediumfile import FileMedium
        self.clock.getNowString.return_value = 'hammertime'
        medium = FileMedium(self.dependencies)
        fmt = Mock()
        fmt.fileExtension = 'prv'
        out = medium.export('provstr', fmt)
        self.listener.exportedToFile.assert_called_with(
            'provenance_hammertime.prv')


    def test_For_PictureCache_format_simply_provides_filename(self):
        from niprov.mediumfile import FileMedium
        from niprov.pictures import PictureCache
        medium = FileMedium(self.dependencies)
        fmt = PictureCache(Mock())
        out = medium.export('provstr', fmt)
        self.listener.exportedToFile.assert_called_with('provstr')
        self.assertEqual(out, 'provstr')






from mock import Mock, patch
from datetime import datetime as dt
from tests.ditest import DependencyInjectionTestBase


class LoggingTests(DependencyInjectionTestBase):

    def setUp(self):
        super(LoggingTests, self).setUp()
        #self.repo.byLocation.return_value = True
        self.opts = Mock()
        self.opts.dryrun = False
        img = Mock()
        img.provenance = {'acquired':dt.now(),
            'subject':'JD', 
            'protocol':'X',
            'technique':'abc',
            'repetition-time':1.0,
            'epi-factor':1.0,
            'magnetization-transfer-contrast':True,
            'diffusion':True,
            'echo-time':123,
            'flip-angle':892,
            'inversion-time':123}
        self.locationFactory.completeString.side_effect = lambda p: p
        self.repo.byLocation.return_value = img
        self.provenancesAdded = []
        def wrapProv(location, transient=False, provenance=False, dependencies=None):
            self.provenancesAdded.append(provenance)
            self.newimg = Mock()
            self.newimg.provenance = {'parentfield':location}
            return self.newimg
        self.dependencies.reconfigureOrGetConfiguration.return_value = self.opts
        patcher = patch('niprov.logging.add')
        self.add = patcher.start()
        self.add.side_effect = wrapProv
        self.addCleanup(patcher.stop)

    def log(self, *args, **kwargs):
        import niprov.logging
        with patch('niprov.logging.inheritFrom') as self.inheritFrom:
            self.inheritFrom.side_effect = lambda p, p2: p
            return niprov.logging.log(*args, dependencies=self.dependencies, opts=self.opts, 
                    **kwargs)

    def test_Returns_img(self):
        parents = ['/p/f1']
        new = '/p/f2'
        trans = 'Something cool'
        out = self.log(new, trans, parents)
        self.assertEqual(out, self.newimg)

    def test_Adds_code_or_logtext(self):
        self.log('new', 'trans', 'old', code='abc', logtext='def')
        self.assertEqual(self.provenancesAdded[0]['code'],'abc')
        self.assertEqual(self.provenancesAdded[0]['logtext'],'def')

    def test_Determines_user_and_logs_them(self):
        self.users.determineUser.return_value = 'paprika'
        self.log('new', 'trans', 'old', user='mononoko')
        self.users.determineUser.assert_called_with('mononoko')
        self.assertEqual(self.provenancesAdded[0]['user'],'paprika')

    def test_Script_added_to_provenance(self):
        parents = ['/p/f1']
        new = '/p/f2'
        trans = 'Something cool'
        script = '/p/test.py'
        self.log(new, trans, parents, script=script)
        self.assertEqual(self.provenancesAdded[0]['script'], script)

    def test_Accepts_and_processes_custom_provenance(self):
        parents = ['/p/f1']
        new = '/p/f2'
        trans = 'Something cool'
        p = {'akey':'avalue'}
        self.log(new, trans, parents, provenance=p)
        self.assertEqual(self.provenancesAdded[0]['akey'], 'avalue')

    def test_Calls_reconfigureOrGetConfiguration_on_dependencies(self):
        outOpts = Mock()
        outOpts.dryrun = True
        self.dependencies.reconfigureOrGetConfiguration.return_value = outOpts
        provenance = self.log(['/p/f1'], 'bla', ['/p/f2'], transient=True)
        self.dependencies.reconfigureOrGetConfiguration.assert_called_with(
            self.opts)

    def test_Can_pass_multiple_new_files(self):
        parents = ['p1','p2']
        new = ['/p/f2','/p/f3']
        trans = 'Something cool'
        self.locationFactory.completeString.side_effect = lambda p: 'l:'+p
        self.log(new, trans, parents)
        self.assertEqual(self.provenancesAdded[0]['parents'], ['l:p1','l:p2'])
        self.assertEqual(self.provenancesAdded[1]['parents'], ['l:p1','l:p2'])

    def test_Add_parent_if_parent_unknown(self):
        self.repo.byLocation.return_value = None
        self.locationFactory.completeString.side_effect = lambda p: 'l:'+p
        provenance = self.log('new', 'trans', 'parentpath')
        self.add.assert_any_call('l:parentpath', dependencies=self.dependencies)
        self.listener.addUnknownParent.assert_called_with('l:parentpath')

    def test_Uses_validated_location_for_parent_lookup(self):
        self.locationFactory.completeString.side_effect = lambda p: 'l:'+p
        provenance = self.log('new', 'trans', 'parentpath')
        self.repo.byLocation.assert_called_with('l:parentpath')

    def test_Inherits_from_parent(self):
        parent = Mock()
        parent.provenance = {'x':123}
        inprov = {'a':89}
        self.repo.byLocation.return_value = parent
        self.locationFactory.completeString.side_effect = lambda p: 'l:'+p
        img = self.log('new', 'trans', 'parentpath', provenance=inprov)
        self.inheritFrom.assert_called_with(inprov, parent.provenance)

    def test_If_parent_unknown_uses_add_return_value(self):
        """In this case the parent provenance should be obtained from
        what add() returns.
        """
        self.locationFactory.completeString.side_effect = lambda p: 'l:'+p
        inprov = {'a':89}
        self.repo.byLocation.return_value = None
        img = self.log('new', 'trans', 'parentpath', provenance=inprov)
        ## add function is mocked to set 'acquired' field to location passed
        self.inheritFrom.assert_called_with(inprov, {'parentfield':'l:parentpath'})

       




#!/usr/bin/python
# -*- coding: UTF-8 -*-
from mock import Mock, sentinel
from tests.ditest import DependencyInjectionTestBase


class DirectMediumTests(DependencyInjectionTestBase):

    def test_Returns_input(self):
        from niprov.mediumdirect import DirectMedium
        exporter = DirectMedium()
        fmt = Mock()
        self.assertEqual(sentinel.one, exporter.export(sentinel.one, fmt))






from mock import Mock, sentinel
from tests.ditest import DependencyInjectionTestBase


class FormatTests(DependencyInjectionTestBase):

    def test_If_serialize_called_with_list_calls_serializeList(self):
        from niprov.format import Format
        exp = Format()
        exp.serializeList = Mock()
        out = exp.serialize([sentinel.p1, sentinel.p2])
        exp.serializeList.assert_called_with([sentinel.p1, sentinel.p2])
        self.assertEqual(out, exp.serializeList())

    def test_If_serialize_called_with_item_calls_serializeSingle(self):
        from niprov.format import Format
        exp = Format()
        exp.serializeSingle = Mock()
        out = exp.serialize(sentinel.p1)
        exp.serializeSingle.assert_called_with(sentinel.p1)
        self.assertEqual(out, exp.serializeSingle())

    def test_If_called_with_dict_calls_serializeStatistics(self):
        from niprov.format import Format
        exp = Format()
        exp.serializeStatistics = Mock()
        out = exp.serialize({'a':'b'})
        exp.serializeStatistics.assert_called_with({'a':'b'})
        self.assertEqual(out, exp.serializeStatistics())

    def test_If_created_with_pipeline_calls_serializePipeline(self):
        from niprov.format import Format
        from niprov.pipeline import Pipeline
        exp = Format(dependencies=self.dependencies)
        exp.serializePipeline = Mock()
        pipe = Pipeline([])
        out = exp.serialize(pipe)
        #self.pipelineFactory.forFile.assert_called_with(img)
        exp.serializePipeline.assert_called_with(pipe)
        self.assertEqual(out, exp.serializePipeline())

    def test_If_created_with_iterable_calls_serializeList(self):
        from niprov.format import Format
        exp = Format()
        exp.serializeList = Mock()
        exp.serializeSingle = Mock()
        class MockQuery(object):
            def __iter__(self):
                pass
        query = MockQuery()
        out = exp.serialize(query)
        exp.serializeList.assert_called_with(query)
        self.assertEqual(out, exp.serializeList())




import unittest
from mock import Mock, patch
from tests.ditest import DependencyInjectionTestBase


class ViewTests(DependencyInjectionTestBase):

    def setUp(self):
        super(ViewTests, self).setUp()
        self.config.dryrun = False
        self.img = Mock()
        self.request = Mock()
        self.request.dependencies = self.dependencies

    def test_latest(self):
        import niprov.views
        out = niprov.views.latest(self.request)
        self.assertEqual(self.repo.latest(), out['images'])

    def test_by_id(self):
        import niprov.views
        self.request.matchdict = {'id':'1a2b3c'}
        out = niprov.views.short(self.request)
        self.repo.byId.assert_called_with('1a2b3c')
        self.assertEqual(self.repo.byId(), out['image'])
        self.query.copiesOf.assert_called_with(out['image'])
        self.assertEqual(self.query.copiesOf(), out['copies'])

    def test_by_full_location(self):
        import niprov.views
        self.request.matchdict = {'host':'her','path':('a','b','c')}
        out = niprov.views.location(self.request)
        self.query.byLocation.assert_called_with('her:/a/b/c')
        self.query.copiesOf.assert_called_with(out['image'])
        self.assertEqual(self.query.byLocation(), out['image'])
        self.assertEqual(self.query.copiesOf(), out['copies'])

    def test_stats(self):
        import niprov.views
        out = niprov.views.stats(self.request)
        self.assertEqual(self.repo.statistics(), out['stats'])

    def test_pipeline_by_id(self):
        import niprov.views
        self.request.matchdict = {'id':'1a2b3c'}
        out = niprov.views.pipeline(self.request)
        self.repo.byId.assert_called_with('1a2b3c')
        self.pipelineFactory.forFile.assert_called_with(self.repo.byId())
        self.assertEqual(self.pipelineFactory.forFile(), out['pipeline'])
        self.assertEqual(out['sid'], '1a2b3c')

    def test_by_project(self):
        import niprov.views
        self.request.matchdict = {'project':'failcow'}
        out = niprov.views.project(self.request)
        self.query.byProject.assert_called_with('failcow')
        self.assertEqual(self.query.byProject(), out['images'])

    def test_by_user(self):
        import niprov.views
        self.request.matchdict = {'user':'failcow'}
        out = niprov.views.user(self.request)
        self.query.byUser.assert_called_with('failcow')
        self.assertEqual(self.query.byUser(), out['images'])

    def test_by_modality(self):
        import niprov.views
        self.request.matchdict = {'modality':'failcow'}
        out = niprov.views.modality(self.request)
        self.query.byModality.assert_called_with('failcow')
        self.assertEqual(self.query.byModality(), out['images'])

    def test_by_subject(self):
        import niprov.views
        self.request.matchdict = {'subject':'janedoe'}
        out = niprov.views.subject(self.request)
        self.query.bySubject.assert_called_with('janedoe')
        self.assertEqual(self.query.bySubject(), out['images'])

    def test_search_provides_searchstring_to_template(self):
        import niprov.views
        self.request.GET = {'text':'hello world'}
        out = niprov.views.search(self.request)
        self.assertEqual('hello world', out['searchtext'])

    def test_search_provides_search_results_to_template(self):
        import niprov.views
        self.request.GET = {'text':'hello world'}
        with patch('niprov.views.searching') as searching:
            out = niprov.views.search(self.request)
            searching.search.assert_called_with('hello world',
                                                self.request.dependencies)
            self.assertEqual(searching.search(), out['images'])

    def test_modalities(self):
        import niprov.views
        out = niprov.views.modalities(self.request)
        self.query.allModalities.assert_called_with()
        self.assertEqual('modalities', out['categoryPlural'])
        self.assertEqual('modality', out['category'])
        self.assertEqual(self.query.allModalities(), out['items'])

    def test_projects(self):
        import niprov.views
        out = niprov.views.projects(self.request)
        self.query.allProjects.assert_called_with()
        self.assertEqual('projects', out['categoryPlural'])
        self.assertEqual('project', out['category'])
        self.assertEqual(self.query.allProjects(), out['items'])

    def test_users(self):
        import niprov.views
        out = niprov.views.users(self.request)
        self.query.allUsers.assert_called_with()
        self.assertEqual('users', out['categoryPlural'])
        self.assertEqual('user', out['category'])
        self.assertEqual(self.query.allUsers(), out['items'])

    def test_If_byId_byLocation_return_None_views_return_404(self):
        from pyramid.httpexceptions import HTTPNotFound
        import niprov.views
        self.request.matchdict = {'host':'her','path':('a','b','c'),
                                    'id':'1a2b3c'}
        self.repo.byId.return_value = None
        self.query.byLocation.return_value = None
        self.assertRaises(HTTPNotFound, niprov.views.short, self.request)
        self.assertRaises(HTTPNotFound, niprov.views.location, self.request)
        self.assertRaises(HTTPNotFound, niprov.views.pipeline, self.request)









import json, copy
from datetime import datetime
from niprov.format import Format


class JsonFormat(Format):
    """Helper to convert provenance data to and from json encoded strings.
    """

    def __init__(self, dependencies):
        super(JsonFormat, self).__init__(dependencies)
        self.fileExtension = 'json'
        self.file = dependencies.getFileFactory()

    def serializeSingle(self, record):
        """
        Convert one provenance item from its native python dict type to
        a json string.

        Args:
            record (dict): The provenance item to convert.

        Returns:
            str: Json version of the provenance.
        """
        flat = self._deflate(record.provenance)
        return json.dumps(flat, cls=DateTimeAwareJSONEncoder)

    def deserialize(self, jsonRecord):
        """
        Convert one provenance item from its json string version to the 
        native python dictionary format.

        Args:
            jsonRecord (str): The provenance item to convert as json-encoded 
                string.

        Returns:
            dict: Python dictionary of the provenance.
        """
        provenance = json.loads(jsonRecord, cls=DateTimeAwareJSONDecoder)
        return self.file.fromProvenance(provenance)

    def serializeList(self, listOfRecords):
        """
        Convert a list of provenance items from its native list of python dict 
        type to a json string.

        Args:
            listOfRecords (list): The provenance items to convert.

        Returns:
            str: Json version of the provenance items.
        """
        flatRecords = [self._deflate(r.provenance) for r in listOfRecords]
        return json.dumps(flatRecords, cls=DateTimeAwareJSONEncoder)

    def deserializeList(self, jsonListOfRecords):
        """
        Convert a list of provenance items from its json string version to the 
        a list of the native python dictionary format.

        Args:
            jsonListOfRecords (str): The provenance items to convert as 
                json-encoded string.

        Returns:
            list: Python list of dictionaries of the provenance.
        """
        provenanceList = json.loads(jsonListOfRecords, cls=DateTimeAwareJSONDecoder)
        return [self.file.fromProvenance(p) for p in provenanceList]

    def _deflate(self, record):
        flatRecord = copy.deepcopy(record)
        def deflateOne(prov):
            if 'args' in prov:
                prov['args'] = [self._strcust(a) for a in prov['args']]
            if 'kwargs' in prov:
                kwargs = prov['kwargs']
                prov['kwargs'] = {k: self._strcust(kwargs[k]) 
                    for k in kwargs.keys()}
            if '_id' in prov:
                del prov['_id']
            return prov
        deflateOne(flatRecord)
        for version in flatRecord.get('_versions', []):
            deflateOne(version)
        return flatRecord

    def _strcust(self, val):
        """Stringify an object that is not of a simple type."""
        if not isinstance(val, (str, unicode, int, float, bool, type(None))):
            return str(val)
        return val

# Taken from http://taketwoprogramming.blogspot.com/2009/06/subclassing-jsonencoder-and-jsondecoder.html

class DateTimeAwareJSONEncoder(json.JSONEncoder):
    """ 
    Converts a python object, where datetime and timedelta objects are converted
    into objects that can be decoded using the DateTimeAwareJSONDecoder.
    """
    def default(self, obj):
        if isinstance(obj, datetime):
            return {
                '__type__' : 'datetime',
                'year' : obj.year,
                'month' : obj.month,
                'day' : obj.day,
                'hour' : obj.hour,
                'minute' : obj.minute,
                'second' : obj.second,
                'microsecond' : obj.microsecond,
            }

        else:
            return json.JSONEncoder.default(self, obj)

class DateTimeAwareJSONDecoder(json.JSONDecoder):
    """ 
    Converts a json string, where datetime and timedelta objects were converted
    into objects using the DateTimeAwareJSONEncoder, back into a python object.
    """

    def __init__(self, **kwargs):
        json.JSONDecoder.__init__(self, object_hook=self.dict_to_object)

    def dict_to_object(self, d):
        if '__type__' not in d:
            return d

        type = d.pop('__type__')
        if type == 'datetime':
            return datetime(**d)
        else:
            # Oops... better put this back together.
            d['__type__'] = type
            return d



from tests.ditest import DependencyInjectionTestBase
from mock import Mock, sentinel
from datetime import datetime

class InheritanceTests(DependencyInjectionTestBase):

    def setUp(self):
        super(InheritanceTests, self).setUp()

    def test_InheritFrom(self):
        from niprov.inheriting import inheritFrom
        parentProv = {'acquired':datetime.now(),
            'subject':'JD', 
            'protocol':'X',
            'technique':'abc',
            'repetition-time':1.0,
            'epi-factor':1.0,
            'magnetization-transfer-contrast':True,
            'diffusion':True,
            'echo-time':123,
            'flip-angle':892,
            'inversion-time':123}
        prov = inheritFrom({'a':1}, parentProv)
        self.assertEqual(prov['acquired'], parentProv['acquired'])
        self.assertEqual(prov['subject'], parentProv['subject'])
        self.assertEqual(prov['protocol'], parentProv['protocol'])
        self.assertEqual(prov['technique'], parentProv['technique'])
        self.assertEqual(prov['repetition-time'], parentProv['repetition-time'])
        self.assertEqual(prov['epi-factor'], parentProv['epi-factor'])
        self.assertEqual(prov['magnetization-transfer-contrast'], 
            parentProv['magnetization-transfer-contrast'])
        self.assertEqual(prov['diffusion'], parentProv['diffusion'])
        self.assertEqual(prov['echo-time'], parentProv['echo-time'])
        self.assertEqual(prov['flip-angle'], parentProv['flip-angle'])
        self.assertEqual(prov['inversion-time'], parentProv['inversion-time'])

    def test_Doesnt_complain_if_parent_is_missing_basic_fields(self):
        from niprov.inheriting import inheritFrom
        prov = inheritFrom({'a':1}, {'b':2})






def inheritFrom(provenance, parentProvenance):
    inheritableFields = [
    'acquired',
    'subject',
    'protocol',
    'technique',
    'repetition-time',
    'epi-factor',
    'magnetization-transfer-contrast',
    'diffusion',
    'echo-time',
    'flip-angle',
    'inversion-time',
    'duration',
    'subject-position',
    'water-fat-shift',
    ]
    for field in inheritableFields:
        if field in parentProvenance:
            provenance[field] = parentProvenance[field]
    return provenance



#!/usr/bin/python
# -*- coding: UTF-8 -*-
from niprov.dependencies import Dependencies


def search(text, dependencies=Dependencies()):
    """
    Search for files with the given text in their provenance fields.

    Args:
        text (str): Words to look for.

    Returns:
        list: List of BaseFile objects
    """
    return dependencies.getRepository().search(text)



from mock import Mock, patch, sentinel
from tests.ditest import DependencyInjectionTestBase
import datetime


class ClockTest(DependencyInjectionTestBase):

    def setUp(self):
        super(ClockTest, self).setUp()

    def test_Can_create_datetimestring(self):
        from niprov.clock import Clock
        now = datetime.datetime(year=1982, month=1, day=2, 
            hour=3, minute=4, second=5, microsecond=6789)
        with patch('niprov.clock.datetime') as datetimeModule:
            datetimeModule.datetime.now.return_value = now
            klok = Clock()
            self.assertEqual(klok.getNowString(), '1982-01-02_03-04-05')






import unittest
from mock import Mock, sentinel
from datetime import datetime
from tests.test_basefile import BaseFileTests


class NiftiTests(BaseFileTests):

    def setUp(self):
        super(NiftiTests, self).setUp()
        self.libs = Mock()
        self.dependencies.getLibraries.return_value = self.libs
        self.img = Mock()
        self.hdr = Mock()
        self.setupNibabel()
        from niprov.nifti import NiftiFile
        self.constructor = NiftiFile
        self.file = NiftiFile(self.path, dependencies=self.dependencies)

    def setupNibabel(self):
        # Extension constructor simply creates a tuple of arguments
        self.img.get_header.return_value = self.hdr
        self.libs.nibabel.nifti1.Nifti1Extension = lambda x,y: ('extension',x,y)
        self.libs.nibabel.load.return_value = self.img
        self.libs.hasDependency.return_value = True

    def test_Attach_method(self):
        self.file.getProvenance = Mock()
        self.file.getProvenance.return_value = 'serial prov'
        self.file.attach('json')
        self.file.getProvenance.assert_called_with('json')
        self.hdr.extensions.append.assert_called_with(('extension','comment', 
            'serial prov'))
        self.img.to_filename.assert_called_with(self.file.path)

    def test_Tells_camera_to_save_snapshot_to_cache(self):
        img = self.libs.nibabel.load.return_value
        data = sentinel.imagedata
        img.get_data.return_value = data
        out = self.file.inspect()
        self.camera.saveSnapshot.assert_called_with(data, for_=self.file)




from unittest import TestCase
from tests.ditest import DependencyInjectionTestBase
from mock import Mock, patch, sentinel
import numpy


class CameraTests(DependencyInjectionTestBase):

    def test_saveSnapshot(self):
        from niprov.camera import Camera
        camera = Camera(self.dependencies)
        camera.takeSnapshot = Mock()
        target = Mock()
        camera.saveSnapshot(target, for_=sentinel.object)
        newPicture = self.pictureCache.new()
        camera.takeSnapshot.assert_called_with(target, on=newPicture)
        self.pictureCache.keep.assert_called_with(newPicture, sentinel.object)

    def test_SaveSnapshot_doesnt_save_if_takeSnapshot_failed(self):
        from niprov.camera import Camera
        camera = Camera(self.dependencies)
        camera.takeSnapshot = Mock()
        camera.takeSnapshot.return_value = False
        target = Mock()
        camera.saveSnapshot(target, for_=sentinel.object)
        self.assertFalse(self.pictureCache.keep.called)

    def test_If_no_matplotlib_installed_takeSnapshot_returns_false(self):
        from niprov.camera import Camera
        camera = Camera(self.dependencies)
        self.libs.hasDependency.return_value = False
        self.libs.pyplot = None
        self.assertFalse(camera.takeSnapshot(numpy.zeros([3,3,3]), Mock()))

    def test_If_exception_during_plotting_returns_False(self):
        from niprov.camera import Camera
        camera = Camera(self.dependencies)
        self.libs.pyplot.subplots.side_effect = ValueError
        self.assertFalse(camera.takeSnapshot(numpy.zeros([3,3,3]), Mock()))

    def test_If_no_exception_during_plotting_returns_True(self):
        from niprov.camera import Camera
        camera = Camera(self.dependencies)
        self.libs.pyplot.subplots.return_value = [Mock(), [Mock()]*3]
        self.assertTrue(camera.takeSnapshot(numpy.zeros([3,3,3]), Mock()))






from tests.ditest import DependencyInjectionTestBase
from mock import Mock


class ObjectFormatTests(DependencyInjectionTestBase):

    def setUp(self):
        super(ObjectFormatTests, self).setUp()

    def test_serialize_item_returns_itself(self):
        from niprov.formatobject import ObjectFormat
        form = ObjectFormat(self.dependencies)
        item = self.aFile()
        out = form.serializeSingle(item)
        self.assertEqual(item, out)

    def test_serialize_list_returns_itself(self):
        from niprov.formatobject import ObjectFormat
        form = ObjectFormat(self.dependencies)
        mylist = [self.aFile(), self.aFile(), self.aFile()]
        out = form.serializeList(mylist)
        self.assertEqual(mylist, out)

    def test_serialize_stats_returns_itself(self):
        from niprov.formatobject import ObjectFormat
        form = ObjectFormat(self.dependencies)
        stats = {'pval':'0.05'}
        out = form.serializeStatistics(stats)
        self.assertEqual(stats, out)

    def test_serialize_pipeline_returns_itself(self):
        from niprov.formatobject import ObjectFormat
        from niprov.pipeline import Pipeline
        form = ObjectFormat(self.dependencies)
        pipe = Pipeline([])
        out = form.serializePipeline(pipe)
        self.assertEqual(pipe, out)

    def aFile(self):
        somefile = Mock()
        somefile.provenance = {'a':'b'}
        return somefile



import unittest
from mock import Mock
from tests.ditest import DependencyInjectionTestBase


class SearchTests(DependencyInjectionTestBase):

    def setUp(self):
        super(SearchTests, self).setUp()

    def test_Search_delegates_to_repository(self):
        from niprov.searching import search
        results = search('one day', dependencies=self.dependencies)
        self.repo.search.assert_called_with('one day')
        self.assertEqual(results, self.repo.search())




import unittest
import os, shutil
from os.path import abspath


class ProvenanceContextApiTests(unittest.TestCase):

    def setUp(self):
        self.dbpath = os.path.expanduser(os.path.join('~','provenance_test.json'))
        if os.path.exists(self.dbpath):
            os.remove(self.dbpath)
        os.mkdir('temp')
        from niprov import ProvenanceContext
        self.provenance = ProvenanceContext()
        self.provenance.config.database_type = 'file'
        self.provenance.config.database_url = self.dbpath

    def tearDown(self):
        shutil.rmtree('temp')

    def touch(self, path):
        with open(path,'w') as tempfile:
            tempfile.write('0')

    def test_Relative_paths(self):
        self.provenance.discover('testdata')
        newfile = 'temp/smoothed.test'
        self.touch(newfile)
        self.provenance.log(newfile, 'test', 'testdata/eeg/stub.cnt')
        img = self.provenance.get().byLocation(newfile)

    def test_Log(self):
        self.provenance.discover('testdata')
        newfile = 'temp/smoothed.test'
        self.touch(newfile)
        parent = os.path.abspath('testdata/eeg/stub.cnt')
        self.provenance.log(newfile, 'test', parent)
        testfpath = os.path.abspath(newfile)
        img = self.provenance.get().byLocation(testfpath)
        self.assertEqual(img.provenance['subject'], 'Jane Doe')
        self.assertEqual(img.provenance['size'], os.path.getsize(newfile))

    def test_Record_with_user(self):
        self.provenance.discover('testdata')
        newfile = 'temp/recorded.test'
        self.touch(newfile)
        parent = os.path.abspath('testdata/eeg/stub.cnt')
        self.provenance.record('echo hallo', newfile, parent, user='007')
        testfpath = os.path.abspath(newfile)
        img = self.provenance.get().byLocation(testfpath)
        self.assertEqual(img.provenance['user'], '007')

    def test_Export_Import(self):
        from niprov.exceptions import UnknownFileError
        self.provenance.discover('testdata')
        discoveredFile = os.path.abspath('testdata/eeg/stub.cnt')
        self.assertIsNotNone(self.provenance.get().byLocation(discoveredFile))
        backupFilepath = self.provenance.backup()
        os.remove(self.dbpath) # get rid of existing data.
        self.assertIsNone(self.provenance.get().byLocation(discoveredFile))
        self.provenance.importp(backupFilepath)
        self.assertIsNotNone(self.provenance.get().byLocation(discoveredFile))

    @unittest.skip("Doesn't work on Travis right now.")
    def test_Attach_provenance_string_in_file_based_on_config(self):
        import nibabel
        self.provenance.config.attach = True
        newfile = os.path.abspath('temp/fileX.nii.gz')
        shutil.copy('testdata/nifti/fieldmap.nii.gz', newfile)
        self.provenance.add(newfile)
        img = nibabel.load(newfile)
        self.assertEqual(img.get_header().extensions.count('comment'), 1)
        self.assertEqual(img.get_header().extensions[0].get_code(), 6)
        content = img.get_header().extensions[0].get_content()
        self.assertIn('"path": "{0}"'.format(newfile), content)

    def test_Approval(self):
        self.provenance.discover('testdata')
        self.provenance.markForApproval(['testdata/parrec/T1.PAR',
                                'testdata/parrec/T2.PAR'])
        imgs = self.provenance.markedForApproval()
        self.provenance.approve('testdata/parrec/T1.PAR')
        imgs = self.provenance.markedForApproval()

    def test_Comparison(self):
        # Given two PARREC images' provenance records
        par1 = self.provenance.add(abspath('testdata/parrec/T1.PAR'))
        par2 = self.provenance.add(abspath('testdata/parrec/T2.PAR'))
        # Comparing them returns a Diff object with methods testing equality
        self.assertFalse(self.provenance.compare(par1, par2).areEqual())
        # Compare() can also be called as a method on the objects themselves,
        # and the Diff object has assert..() methods that raise AssertionErrors
        msgRegExp = ".*echo-time.*2\.08.*80\.0.*"
        with self.assertRaisesRegexp(AssertionError, msgRegExp):
            par1.compare(par2).assertEqualProtocol()

    def test_Search(self):
        x1 = self.provenance.add('x1', transient=True,
            provenance={'transformation':'needle and thread'})
        x2 = self.provenance.add('x2/needle.y', transient=True, 
            provenance={'transformation':'needle and thread'})
        x3 = self.provenance.add('x3', transient=True, 
            provenance={'transformation':'hammer and tongs'})
        results = self.provenance.search('needle')
        self.assertEqual(len(results), 2)
        self.assertEqual(x2.provenance['id'], results[0].provenance['id'])
        self.assertEqual(x1.provenance['id'], results[1].provenance['id'])

    def test_GetModalities(self):
        self.provenance.discover('testdata')
        modalities = self.provenance.get().allModalities()
        self.assertIn('MRI', modalities)
        self.assertIn('DWI', modalities)
        self.assertIn('EEG', modalities)

    def test_Version_history(self):
        self.provenance.add('f', transient=True, provenance={'a':1})
        img = self.provenance.get().byLocation('f')
        self.assertEqual(1, img.provenance['a'])
        self.provenance.add('f', transient=True, provenance={'a':2})
        img = self.provenance.get().byLocation('f')
        self.assertEqual(2, img.provenance['a'])
        self.assertEqual(1, img.versions[-1]['a'])
        self.provenance.add('f', transient=True, provenance={'a':3})
        img = self.provenance.get().byLocation('f')
        self.assertEqual(3, img.provenance['a'])
        self.assertEqual(2, img.versions[-1]['a'])
        self.assertEqual(1, img.versions[-2]['a'])

    def test_If_no_parent_provided_found_copy_considered_parent(self):
        self.provenance.add('testdata/eeg/stub.cnt')
        self.touch('temp/orig.f')
        self.provenance.log('temp/orig.f', 'op1', 'testdata/eeg/stub.cnt')
        shutil.copy('temp/orig.f', 'temp/copy.f')
        self.touch('temp/child.f')
        child = self.provenance.log('temp/child.f', 'op2', 'temp/copy.f')
        self.assertEqual(child.provenance['subject'], 'Jane Doe')
        copy = self.provenance.get().byLocation('temp/copy.f')
        self.assertIn('temp/orig.f', copy.provenance['parents'][0])
 

if __name__ == '__main__':
    unittest.main()




import unittest
import numpy
from mock import Mock, sentinel
from datetime import datetime
from tests.test_basefile import BaseFileTests


class ParrecTests(BaseFileTests):

    def setUp(self):
        super(ParrecTests, self).setUp()
        self.libs = Mock()
        self.dependencies.getLibraries.return_value = self.libs
        self.setupNibabel()
        from niprov.parrec import ParrecFile
        self.constructor = ParrecFile
        self.file = ParrecFile(self.path, dependencies=self.dependencies)

    def test_Gets_basic_info_from_nibabel_and_returns_it(self):
        out = self.file.inspect()
        self.assertEqual(out['subject'], 'John Doeish')
        self.assertEqual(out['protocol'], 'T1 SENSE')
        self.assertEqual(out['acquired'], datetime(2014, 8, 5, 11, 27, 34))

    def test_Gets_dimensions(self):
        out = self.file.inspect()
        self.assertEqual(out['dimensions'], [80,80,10])

    def test_Gets_advanced_fields(self):
        out = self.file.inspect()
        self.assertEqual(out['technique'], 'T1TFE')
        self.assertEqual(out['repetition-time'], 4.364)
        self.assertEqual(out['field-of-view'], [130., 100., 154.375])
        self.assertEqual(out['epi-factor'], 1)
        self.assertEqual(out['magnetization-transfer-contrast'], False)
        self.assertEqual(out['diffusion'], False)
        self.assertEqual(out['duration'], 65)
        self.assertEqual(out['subject-position'], 'Head First Supine')
        self.assertEqual(out['water-fat-shift'], 1.117)
        # per-image
        self.assertEqual(out['slice-thickness'], 10.0)
        self.assertEqual(out['slice-orientation'], 1)
        self.assertEqual(out['echo-time'], 2.0800000000000001)
        self.assertEqual(out['flip-angle'], 8.0)
        self.assertEqual(out['inversion-time'], 0.0)

    def test_getProtocolFields(self):
        protocol = self.file.getProtocolFields()
        self.assertIn('repetition-time', protocol)
        self.assertIn('echo-time', protocol)

    def test_multiple_TRs(self):
        img = self.libs.nibabel.load.return_value
        img.header.general_info['repetition_time'] = numpy.array([130, 450])
        self.libs.nibabel.load.return_value = img
        out = self.file.inspect()
        self.assertEqual(out['repetition-time'], [130, 450])

    def test_Tells_camera_to_save_snapshot_to_cache(self):
        img = self.libs.nibabel.load.return_value
        data = sentinel.imagedata
        img.get_data.return_value = data
        out = self.file.inspect()
        self.camera.saveSnapshot.assert_called_with(data, for_=self.file)

    def test_Determines_modality(self):
        out = self.file.inspect()
        self.assertEqual(out['modality'], 'MRI')

    def test_Determines_modality_for_diffusion(self):
        img = self.libs.nibabel.load.return_value
        img.header.general_info['diffusion'] = 1
        self.libs.nibabel.load.return_value = img
        out = self.file.inspect()
        self.assertEqual(out['modality'], 'DWI')

    def test_Preserves_modality_if_inherited(self):
        pass # Doesn't have to preserve

    def setupNibabel(self):
        import numpy
        img = Mock()
        img.header.general_info = {
             'acq_nr': 6,
             'angulation': numpy.array([-1.979,  0.546,  0.019]),
             'diffusion': 0,
             'diffusion_echo_time': 0.0,
             'dyn_scan': 0,
             'epi_factor': 1,
             'exam_date': '2014.08.05 / 11:27:34',
             'exam_name': 'test',
             'flow_compensation': 0,
             'fov': numpy.array([ 130.   ,  100.   ,  154.375]),
             'max_cardiac_phases': 1,
             'max_diffusion_values': 1,
             'max_dynamics': 1,
             'max_echoes': 1,
             'max_gradient_orient': 1,
             'max_mixes': 1,
             'max_slices': 10,
             'mtc': 0,
             'nr_label_types': 0,
             'off_center': numpy.array([-18.805,  22.157, -17.977]),
             'patient_name': 'John Doeish',
             'patient_position': 'Head First Supine',
             'phase_enc_velocity': numpy.array([ 0.,  0.,  0.]),
             'prep_direction': 'Right-Left',
             'presaturation': 0,
             'protocol_name': 'T1 SENSE',
             'recon_nr': 1,
             'repetition_time': 4.364,
             'scan_duration': 65.0,
             'scan_mode': '3D',
             'scan_resolution': numpy.array([76, 62]),
             'series_type': 'Image   MRSERIES',
             'spir': 0,
             'tech': 'T1TFE',
             'water_fat_shift': 1.117}
        img.header.image_defs = numpy.array([ (1, 1, 1, 1, 0, 2, 0, 16, 81, [80, 80], 0.0, 1.26032, 2.84925e-05, 133, 231, [-1.98, 0.55, 0.02], [-18.79, -22.82, -16.42], 10.0, 0.0, 0, 1, 0, 2, [1.912, 1.912], 2.08, 0.0, 0.0, 0.0, 1, 8.0, 0, 0, 0, 7, 0.0, 1, 1, '7', '0', [0.0, 0.0, 0.0], 1),
       (2, 1, 1, 1, 0, 2, 1, 16, 81, [80, 80], 0.0, 1.26032, 2.84925e-05, 294, 512, [-1.98, 0.55, 0.02], [-18.79, -12.82, -16.77], 10.0, 0.0, 0, 1, 0, 2, [1.912, 1.912], 2.08, 0.0, 0.0, 0.0, 1, 8.0, 0, 0, 0, 7, 0.0, 1, 1, '7', '0', [0.0, 0.0, 0.0], 1),
       (3, 1, 1, 1, 0, 2, 2, 16, 81, [80, 80], 0.0, 1.26032, 2.84925e-05, 427, 742, [-1.98, 0.55, 0.02], [-18.8, -2.83, -17.11], 10.0, 0.0, 0, 1, 0, 2, [1.912, 1.912], 2.08, 0.0, 0.0, 0.0, 1, 8.0, 0, 0, 0, 7, 0.0, 1, 1, '7', '0', [0.0, 0.0, 0.0], 1)], 
      dtype=[('slice number', '<i8'), ('echo number', '<i8'), ('dynamic scan number', '<i8'), ('cardiac phase number', '<i8'), ('image_type_mr', '<i8'), ('scanning sequence', '<i8'), ('index in REC file', '<i8'), ('image pixel size', '<i8'), ('scan percentage', '<i8'), ('recon resolution', '<i8', (2,)), ('rescale intercept', '<f8'), ('rescale slope', '<f8'), ('scale slope', '<f8'), ('window center', '<i8'), ('window width', '<i8'), ('image angulation', '<f8', (3,)), ('image offcentre', '<f8', (3,)), ('slice thickness', '<f8'), ('slice gap', '<f8'), ('image_display_orientation', '<i8'), ('slice orientation', '<i8'), ('fmri_status_indication', '<i8'), ('image_type_ed_es', '<i8'), ('pixel spacing', '<f8', (2,)), ('echo_time', '<f8'), ('dyn_scan_begin_time', '<f8'), ('trigger_time', '<f8'), ('diffusion_b_factor', '<f8'), ('number of averages', '<i8'), ('image_flip_angle', '<f8'), ('cardiac frequency', '<i8'), ('minimum RR-interval', '<i8'), ('maximum RR-interval', '<i8'), ('TURBO factor', '<i8'), ('Inversion delay', '<f8'), ('diffusion b value number', '<i8'), ('gradient orientation number', '<i8'), ('contrast type', 'S30'), ('diffusion anisotropy type', 'S30'), ('diffusion', '<f8', (3,)), ('label type', '<i8')])
        img.shape = (80,80,10)
        self.libs.nibabel.load.return_value = img
        self.libs.hasDependency.return_value = True




